"""This module contains the general information for FirmwareChassisPackModImpact ManagedObject."""

from ...ucsmo import ManagedObject
from ...ucscoremeta import MoPropertyMeta, MoMeta
from ...ucsmeta import VersionMeta


class FirmwareChassisPackModImpactConsts:
    pass


class FirmwareChassisPackModImpact(ManagedObject):
    """This is FirmwareChassisPackModImpact class."""

    consts = FirmwareChassisPackModImpactConsts()
    naming_props = set(['keyDn'])

    mo_meta = MoMeta("FirmwareChassisPackModImpact", "firmwareChassisPackModImpact", "fw-sys-ChassisPackModImpact-[key_dn]", VersionMeta.Version323a, "InputOutput", 0x3f, [], ["admin", "ls-config-policy", "ls-server-policy"], [], [], [None])

    prop_meta = {
        "chassis_profile_dn": MoPropertyMeta("chassis_profile_dn", "ChassisProfileDn", "string", VersionMeta.Version323a, MoPropertyMeta.READ_ONLY, None, 0, 256, None, [], []),
        "child_action": MoPropertyMeta("child_action", "childAction", "string", VersionMeta.Version323a, MoPropertyMeta.INTERNAL, 0x2, None, None, r"""((deleteAll|ignore|deleteNonPresent),){0,2}(deleteAll|ignore|deleteNonPresent){0,1}""", [], []),
        "dn": MoPropertyMeta("dn", "dn", "string", VersionMeta.Version323a, MoPropertyMeta.READ_ONLY, 0x4, 0, 256, None, [], []),
        "key_dn": MoPropertyMeta("key_dn", "keyDn", "string", VersionMeta.Version323a, MoPropertyMeta.NAMING, 0x8, 1, 510, None, [], []),
        "maint_policy_dn": MoPropertyMeta("maint_policy_dn", "maintPolicyDn", "string", VersionMeta.Version323a, MoPropertyMeta.READ_ONLY, None, 0, 256, None, [], []),
        "pn_dn": MoPropertyMeta("pn_dn", "pnDn", "string", VersionMeta.Version323a, MoPropertyMeta.READ_ONLY, None, 0, 256, None, [], []),
        "reboot_policy": MoPropertyMeta("reboot_policy", "rebootPolicy", "string", VersionMeta.Version323a, MoPropertyMeta.READ_ONLY, None, 0, 510, None, [], []),
        "rn": MoPropertyMeta("rn", "rn", "string", VersionMeta.Version323a, MoPropertyMeta.READ_ONLY, 0x10, 0, 256, None, [], []),
        "sacl": MoPropertyMeta("sacl", "sacl", "string", VersionMeta.Version323a, MoPropertyMeta.READ_ONLY, None, None, None, r"""((none|del|mod|addchild|cascade),){0,4}(none|del|mod|addchild|cascade){0,1}""", [], []),
        "status": MoPropertyMeta("status", "status", "string", VersionMeta.Version323a, MoPropertyMeta.READ_WRITE, 0x20, None, None, r"""((removed|created|modified|deleted),){0,3}(removed|created|modified|deleted){0,1}""", [], []),
    }

    prop_map = {
        "ChassisProfileDn": "chassis_profile_dn", 
        "childAction": "child_action", 
        "dn": "dn", 
        "keyDn": "key_dn", 
        "maintPolicyDn": "maint_policy_dn", 
        "pnDn": "pn_dn", 
        "rebootPolicy": "reboot_policy", 
        "rn": "rn", 
        "sacl": "sacl", 
        "status": "status", 
    }

    def __init__(self, parent_mo_or_dn, key_dn, **kwargs):
        self._dirty_mask = 0
        self.key_dn = key_dn
        self.chassis_profile_dn = None
        self.child_action = None
        self.maint_policy_dn = None
        self.pn_dn = None
        self.reboot_policy = None
        self.sacl = None
        self.status = None

        ManagedObject.__init__(self, "FirmwareChassisPackModImpact", parent_mo_or_dn, **kwargs)



"""This module contains the meta information of ApeSetVmediaMounts ExternalMethod."""

from ..ucscoremeta import MethodMeta, MethodPropertyMeta

method_meta = MethodMeta("ApeSetVmediaMounts", "apeSetVmediaMounts", "Version142b")

prop_meta = {
    "cookie": MethodPropertyMeta("Cookie", "cookie", "Xs:string", "Version142b", "InputOutput", False),
    "in_chassis_id": MethodPropertyMeta("InChassisId", "inChassisId", "Xs:unsignedInt", "Version142b", "Input", False),
    "in_count": MethodPropertyMeta("InCount", "inCount", "Xs:unsignedInt", "Version142b", "Input", False),
    "in_server_instance_id": MethodPropertyMeta("InServerInstanceId", "inServerInstanceId", "Xs:unsignedInt", "Version142b", "Input", False),
    "in_slot_id": MethodPropertyMeta("InSlotId", "inSlotId", "Xs:unsignedInt", "Version142b", "Input", False),
    "in_v_media_set": MethodPropertyMeta("InVMediaSet", "inVMediaSet", "ConfigSet", "Version142b", "Input", True),
}

prop_map = {
    "cookie": "cookie",
    "inChassisId": "in_chassis_id",
    "inCount": "in_count",
    "inServerInstanceId": "in_server_instance_id",
    "inSlotId": "in_slot_id",
    "inVMediaSet": "in_v_media_set",
}



"""This module contains the general information for AdaptorHostMgmtCap ManagedObject."""

from ...ucsmo import ManagedObject
from ...ucscoremeta import MoPropertyMeta, MoMeta
from ...ucsmeta import VersionMeta


class AdaptorHostMgmtCapConsts:
    MODE_FULL = "full"
    MODE_PARTIAL = "partial"
    OPER_POWER_REQUIREMENT_FULL = "full"
    OPER_POWER_REQUIREMENT_NONE = "none"
    OPER_POWER_REQUIREMENT_STANDBY = "standby"
    PREBOOT_EFI = "EFI"
    PREBOOT_PNU_OS = "PnuOS"
    PREBOOT_NONE = "none"
    PRESENCE_CIMC = "cimc"
    PRESENCE_HOST = "host"
    PRESENCE_UNSPECIFIED = "unspecified"
    REBOOT_ACTION_ON_DESTRUCTIVE_ADAPTOR = "adaptor"
    REBOOT_ACTION_ON_DESTRUCTIVE_HOST = "host"
    REBOOT_ACTION_ON_DESTRUCTIVE_NONE = "none"


class AdaptorHostMgmtCap(ManagedObject):
    """This is AdaptorHostMgmtCap class."""

    consts = AdaptorHostMgmtCapConsts()
    naming_props = set([])

    mo_meta = MoMeta("AdaptorHostMgmtCap", "adaptorHostMgmtCap", "host-mgmt", VersionMeta.Version101e, "InputOutput", 0x3ff, [], ["read-only"], ['adaptorFruCapProvider'], [], ["Get"])

    prop_meta = {
        "child_action": MoPropertyMeta("child_action", "childAction", "string", VersionMeta.Version101e, MoPropertyMeta.INTERNAL, 0x2, None, None, r"""((deleteAll|ignore|deleteNonPresent),){0,2}(deleteAll|ignore|deleteNonPresent){0,1}""", [], []),
        "dn": MoPropertyMeta("dn", "dn", "string", VersionMeta.Version101e, MoPropertyMeta.READ_ONLY, 0x4, 0, 256, None, [], []),
        "mode": MoPropertyMeta("mode", "mode", "string", VersionMeta.Version101e, MoPropertyMeta.READ_WRITE, 0x8, None, None, None, ["full", "partial"], []),
        "oper_power_requirement": MoPropertyMeta("oper_power_requirement", "operPowerRequirement", "string", VersionMeta.Version101e, MoPropertyMeta.READ_WRITE, 0x10, None, None, None, ["full", "none", "standby"], []),
        "preboot": MoPropertyMeta("preboot", "preboot", "string", VersionMeta.Version101e, MoPropertyMeta.READ_WRITE, 0x20, None, None, None, ["EFI", "PnuOS", "none"], []),
        "presence": MoPropertyMeta("presence", "presence", "string", VersionMeta.Version202m, MoPropertyMeta.READ_WRITE, 0x40, None, None, None, ["cimc", "host", "unspecified"], []),
        "reboot_action_on_destructive": MoPropertyMeta("reboot_action_on_destructive", "rebootActionOnDestructive", "string", VersionMeta.Version101e, MoPropertyMeta.READ_WRITE, 0x80, None, None, None, ["adaptor", "host", "none"], []),
        "rn": MoPropertyMeta("rn", "rn", "string", VersionMeta.Version101e, MoPropertyMeta.READ_ONLY, 0x100, 0, 256, None, [], []),
        "sacl": MoPropertyMeta("sacl", "sacl", "string", VersionMeta.Version302c, MoPropertyMeta.READ_ONLY, None, None, None, r"""((none|del|mod|addchild|cascade),){0,4}(none|del|mod|addchild|cascade){0,1}""", [], []),
        "status": MoPropertyMeta("status", "status", "string", VersionMeta.Version101e, MoPropertyMeta.READ_WRITE, 0x200, None, None, r"""((removed|created|modified|deleted),){0,3}(removed|created|modified|deleted){0,1}""", [], []),
    }

    prop_map = {
        "childAction": "child_action", 
        "dn": "dn", 
        "mode": "mode", 
        "operPowerRequirement": "oper_power_requirement", 
        "preboot": "preboot", 
        "presence": "presence", 
        "rebootActionOnDestructive": "reboot_action_on_destructive", 
        "rn": "rn", 
        "sacl": "sacl", 
        "status": "status", 
    }

    def __init__(self, parent_mo_or_dn, **kwargs):
        self._dirty_mask = 0
        self.child_action = None
        self.mode = None
        self.oper_power_requirement = None
        self.preboot = None
        self.presence = None
        self.reboot_action_on_destructive = None
        self.sacl = None
        self.status = None

        ManagedObject.__init__(self, "AdaptorHostMgmtCap", parent_mo_or_dn, **kwargs)



"""This module contains the meta information of ApeSetFlexFlashVirtualRaidInformation ExternalMethod."""

from ..ucscoremeta import MethodMeta, MethodPropertyMeta

method_meta = MethodMeta("ApeSetFlexFlashVirtualRaidInformation", "apeSetFlexFlashVirtualRaidInformation", "Version142b")

prop_meta = {
    "cookie": MethodPropertyMeta("Cookie", "cookie", "Xs:string", "Version142b", "InputOutput", False),
    "in_conrtoller_id": MethodPropertyMeta("InConrtollerId", "inConrtollerId", "Xs:unsignedInt", "Version142b", "Input", False),
    "in_ip_addr": MethodPropertyMeta("InIpAddr", "inIpAddr", "Xs:unsignedInt", "Version142b", "Input", False),
    "in_raid_health": MethodPropertyMeta("InRaidHealth", "inRaidHealth", "Xs:unsignedInt", "Version142b", "Input", False),
    "in_raid_state": MethodPropertyMeta("InRaidState", "inRaidState", "Xs:unsignedInt", "Version142b", "Input", False),
}

prop_map = {
    "cookie": "cookie",
    "inConrtollerId": "in_conrtoller_id",
    "inIpAddr": "in_ip_addr",
    "inRaidHealth": "in_raid_health",
    "inRaidState": "in_raid_state",
}



# Copyright 2015 Cisco Systems, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from nose.tools import *


def test_import_ucsmsdk():
    import ucsmsdk


def test_import_ucsbasetype():
    from ucsmsdk import ucsbasetype


def test_import_ucsconstants():
    from ucsmsdk import ucsconstants


def test_import_ucscore():
    from ucsmsdk import ucscore


def test_import_ucscoremeta():
    from ucsmsdk import ucscoremeta


def test_import_ucscoreutils():
    from ucsmsdk import ucscoreutils


def test_import_ucsdriver():
    from ucsmsdk import ucsdriver


def test_import_ucseventhandler():
    from ucsmsdk import ucseventhandler


def test_import_ucsexception():
    from ucsmsdk import ucsexception


def test_import_ucsfilter():
    from ucsmsdk import ucsfilter


def test_import_ucsfiltertype():
    from ucsmsdk import ucsfiltertype


def test_import_ucsgenutils():
    from ucsmsdk import ucsgenutils


def test_import_ucshandle():
    from ucsmsdk import ucshandle


def test_import_ucsmeta():
    from ucsmsdk import ucsmeta


def test_import_ucsmethod():
    from ucsmsdk import ucsmethod


def test_import_ucsmethodfactory():
    from ucsmsdk import ucsmethodfactory


def test_import_ucsmo():
    from ucsmsdk import ucsmo


def test_import_ucssession():
    from ucsmsdk import ucssession


def test_import_ucsxmlcodec():
    from ucsmsdk import ucsxmlcodec


def test_import_utils():
    import ucsmsdk.utils


def test_import_ccoimage():
    from ucsmsdk.utils import ccoimage


def test_import_comparesyncmo():
    from ucsmsdk.utils import comparesyncmo


def test_import_converttopython():
    from ucsmsdk.utils import converttopython


def test_import_ucsbackup():
    from ucsmsdk.utils import ucsbackup


def test_import_ucsguilaunch():
    from ucsmsdk.utils import ucsguilaunch


def test_import_ucskvmlaunch():
    from ucsmsdk.utils import ucskvmlaunch


def test_import_ucstechsupport():
    from ucsmsdk.utils import ucstechsupport





# Copyright 2013 Cisco Systems, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
This module is responsible for event handling of the events exposed by
UCSM server.
"""

from __future__ import print_function

try:
    from Queue import Queue
except:
    from queue import Queue

from threading import Condition, Lock, Thread
import datetime
import logging
import time

from . import ucsmo
from . import ucscoreutils
from . import ucsxmlcodec as xc
from .ucsexception import UcsWarning
from .ucsexception import UcsValidationException

log = logging.getLogger('ucs')


class MoChangeEvent(object):
    """
    This class provides structure to save an event generated for any change,
    its associated managed object and property change list.
    This functionality is used during add_event_handler.
    """

    def __init__(self, event_id=None, mo=None, change_list=None):
        self.event_id = event_id
        self.mo = mo
        self.change_list = change_list


class WatchBlock(object):
    """
    This class handles the functionality about the Event Handling/Watch block.
    enqueue/dequeue fuctionality of events is handled by this class.
    This functionality is used during add_event_handler.
    """

    def __init__(self, params, fmce, capacity, callback):
        self.fmce = fmce
        self.callback = callback
        self.capacity = capacity
        self.params = params
        self.overflow = False
        self.error_code = 0
        self.event_q = Queue()  # infinite size Queue

    def dequeue(self, miliseconds_timeout):
        """Internal method to dequeue the events."""
        while True:
            if self.error_code != 0:
                log.debug("queue error:" + str(self.error_code))
                return None
            if not self.event_q.empty():
                mo_chg_event = self.event_q.get_nowait()
                return mo_chg_event
            else:
                return None

    def enqueue(self, cmce):
        """Internal method to enqueue the events."""
        if self.event_q.maxsize < self.capacity:
            self.event_q.put(cmce)
        else:
            self.overflow = True

    def queue_size(self):
        return self.event_q.qsize()

    def dequeue_default_callback(self, mce):
        """Default callback method."""
        tab_size = 8
        print("\n")
        print('EventId'.ljust(tab_size * 2) + ':' + str(mce.event_id))
        print('ChangeList'.ljust(tab_size * 2) + ':' + str(mce.change_list))
        print('ClassId'.ljust(tab_size * 2) + ':' + str(mce.mo.get_class_id()))
        print('MoDn'.ljust(tab_size * 2) + ':' + str(mce.mo.dn))


class UcsEventHandle(object):
    """This class provides api to add and remove event handler."""

    def __init__(self, handle):
        self._handle = handle
        self._lock_object = None
        self._wbs = []
        self._wbs_lock = Lock()
        self._enqueue_thread = None
        self._condition = Condition()
        self._event_chan_resp = None
        self._dequeue_thread = None
        self._lowest_timeout = None
        self._wb_to_remove = []

    def _process_method_vessel(self, root, elems):
        for in_stimuli in root:
            for cmce in in_stimuli:
                for in_config in cmce:
                    for mo_elem in in_config:
                        elems.append(
                            (mo_elem, cmce.attrib.get('inEid')))
        return elems

    def _process_mo_change_event(self, root, elems):
        for in_config in root:
            for mo_elem in in_config:
                elems.append(mo_elem)
        return elems

    def _get_mo_elem(self, xml_str):
        """
        Internal method to extract mo elements from xml string
        """

        root = xc.extract_root_elem(xml_str)
        mo_elems = []
        if root.tag == "methodVessel":
            mo_elems = self._process_method_vessel(root, mo_elems)
        elif root.tag == "configMoChangeEvent":
            mo_elems = self._process_mo_change_event(root, mo_elems)
        return mo_elems

    def _can_enqueue(self):
        return self._event_chan_resp and len(
            self._wbs) and (
            self._handle.cookie is not None)

    def _notify_to_dequeue(self):
        with self._condition:
            self._condition.notify()

    def _process_event_channel_resp(self, resp):
        enqueued = False

        for mo_elem in self._get_mo_elem(resp):
            gmo = ucsmo.generic_mo_from_xml_elem(mo_elem[0])
            mce = MoChangeEvent(event_id=mo_elem[1],
                                mo=gmo.to_mo(),
                                change_list=gmo.properties.keys())

            for watch_block in self._wbs:
                if watch_block.fmce(mce):
                    watch_block.enqueue(mce)
                    enqueued = True

        if enqueued:
            self._notify_to_dequeue()

    def _enqueue_function(self):
        """
        Internal method used by add_event_handler.
        Provides functionality of enqueue/dequeue of the events and
        triggering callbacks.
        """

        try:
            xml_query = '<eventSubscribe cookie="%s"/>' % self._handle.cookie
            self._event_chan_resp = self._handle.post_xml(
                xml_str=xml_query.encode(), read=False)
        except Exception:
            raise

        try:
            while self._can_enqueue():
                resp = self._event_chan_resp.readline()
                resp = self._event_chan_resp.read(int(resp))
                self._process_event_channel_resp(resp)

            if len(self._wbs) == 0:
                self._condition.acquire()
                self._condition.notify()
                self._condition.release()
        except:
            raise

    def _thread_enqueue_start(self):
        """
        Internal method to start the enqueue thread which adds the events in
        an internal queue.
        """

        self._enqueue_thread = Thread(name="enqueue_thread",
                                      target=self._enqueue_function)
        self._enqueue_thread.daemon = True
        self._enqueue_thread.start()

    def _time_left(self, watch_block):
        timeout_sec = watch_block.params["timeout_sec"]
        start_time = watch_block.params["start_time"]
        time_diff = datetime.datetime.now() - start_time
        if time_diff.seconds < timeout_sec:
            return timeout_sec - time_diff.seconds
        else:
            return 0
        # return 2147483647

    def _dequeue_mce(self, time_left, watch_block):
        if time_left and time_left > 0:
            if self._lowest_timeout is None or \
                    self._lowest_timeout > time_left:
                self._lowest_timeout = time_left
            mce = watch_block.dequeue(time_left)
        else:
            mce = watch_block.dequeue(2147483647)

        return mce

    def _get_ucs_prop_name(self, mo, python_prop):
        if isinstance(mo, ucsmo.GenericMo):
            return python_prop
        elif python_prop in mo.prop_meta:
            # return the Ucs equivalent of this property
            # so `usr_lbl` will be returned as `usrLbl`
            return mo.prop_meta[python_prop].xml_attribute
        else:
            # we do not know about this property
            # return the same back - no other option
            return python_prop

    def _get_prop_value(self, mo, python_prop):
        if isinstance(mo, ucsmo.GenericMo):
            return mo.properties[python_prop]
        elif python_prop in mo.prop_meta:
            return getattr(mo, python_prop)
        elif python_prop in vars(mo):
            # we do not know about this property
            # but the property seems to be available
            # in the mo. so return mo[prop]
            return getattr(mo, python_prop)
        else:
            # Not sure what we can do!!
            return None

    def _should_skip_prop_match(self, prop, change_list):
        # if prop is not a part of change_list
        # skip processing the property
        return change_list and prop not in change_list

    def _is_property_in_success_values(self, value, success_values):
        return len(success_values) > 0 and (value in success_values)

    def _prop_val_match(self, mo, prop, success_values,
                        change_list=None):
        ucs_prop = self._get_ucs_prop_name(mo, prop)
        prop_val = self._get_prop_value(mo, prop)

        if self._should_skip_prop_match(ucs_prop, change_list):
            return False

        return self._is_property_in_success_values(prop_val, success_values)

    def _invoke_callback_and_set_done(self, wb, mce):
        if wb.callback:
            ctxt = wb.params['context']
            if ctxt:
                ctxt["done"] = True
            wb.callback(mce)

    def _dequeue_mo_prop_poll(self, mo, prop, poll_sec, watch_block,
                              timeout_sec=None, time_left=None):

        success_value = watch_block.params["success_value"]

        if not success_value or len(success_value) < 1:
            raise ValueError("success_value is missing.")

        pmo = self._handle.query_dn(mo.dn)
        if pmo is None:
            UcsWarning('Mo ' + pmo.dn + ' not found.')
            return

        if timeout_sec is not None and time_left is not None and time_left > 0:
            if time_left < poll_sec:
                poll_sec = timeout_sec - time_left

        if self._lowest_timeout is None or self._lowest_timeout > poll_sec:
            self._lowest_timeout = poll_sec

        if self._prop_val_match(pmo, prop, success_value):
            mce = MoChangeEvent(mo=pmo)
            self._invoke_callback_and_set_done(watch_block, mce)
            self._wb_to_remove.append(watch_block)

    def _dequeue_mo_prop_event(self, prop, watch_block, time_left=None):

        success_value = watch_block.params["success_value"]

        if not success_value or len(success_value) < 1:
            raise ValueError("success_value is missing.")

        # dequeue mce
        mce = self._dequeue_mce(time_left, watch_block)
        if mce is None:
            return

        # checks if prop value exist in success value(s)
        attributes = mce.change_list
        if self._prop_val_match(mce.mo, prop, success_value, attributes):
            self._invoke_callback_and_set_done(watch_block, mce)
            self._wb_to_remove.append(watch_block)

    def _dequeue_mo_until_removed(self, watch_block, time_left=None):

        # dequeue mce
        mce = self._dequeue_mce(time_left, watch_block)
        if mce is None:
            return

        if watch_block.callback is not None:
            watch_block.callback(mce)

        # watch mo until gets deleted
        if mce.mo.status == "deleted":
            self._wb_to_remove.append(watch_block)

    def _dequeue_all_class_id(self, watch_block, time_left=None):

        # dequeue mce
        mce = self._dequeue_mce(time_left, watch_block)
        if mce is not None and watch_block.callback is not None:
            watch_block.callback(mce)

    def _dequeue_function(self):
        """
        Internal method to dequeue to events.
        """
        while len(self._wbs):
            self._lowest_timeout = None
            self._wb_to_remove = []

            try:
                for watch_block in self._wbs:
                    mo = watch_block.params["managed_object"]
                    prop = watch_block.params["prop"]
                    poll_sec = watch_block.params["poll_sec"]
                    timeout_sec = watch_block.params["timeout_sec"]

                    # checks if watch_block is not timed out, else remove
                    time_left = None
                    if timeout_sec is not None:
                        time_left = self._time_left(watch_block)
                        if time_left <= 0:
                            self._wb_to_remove.append(watch_block)
                            continue
                    if self._lowest_timeout is None:
                        self._lowest_timeout = time_left
                    elif self._lowest_timeout < time_left:
                        self._lowest_timeout = time_left


                    # Dequeue any change events for a specified watch_block
                    # poll for mo. Not to monitor event.
                    if poll_sec is not None and mo is not None:
                        self._dequeue_mo_prop_poll(mo, prop, poll_sec,
                                                   watch_block, timeout_sec,
                                                   time_left)
                    elif mo is not None:
                        while watch_block.queue_size() > 0:
                            # watch mo until prop_val changed to desired value
                            if prop is not None:
                                self._dequeue_mo_prop_event(prop, watch_block,
                                                            time_left)
                            # watch mo until it is removed
                            else:
                                self._dequeue_mo_until_removed(watch_block,
                                                               time_left)
                    elif mo is None:
                        while watch_block.queue_size() > 0:
                            # watch all event or specific to class_id
                            self._dequeue_all_class_id(watch_block, time_left)
            except Exception as e:
                log.info(str(e))
                self._wb_to_remove.append(watch_block)

            # remove any watch blocks in to_remove list
            self._process_wb_remove_list()

            # wait for more events only if watch_block exists
            if len(self._wbs):
                with self._condition:
                    self._condition.wait(self._lowest_timeout)
        return

    def _process_wb_remove_list(self):
        if len(self._wb_to_remove) == 0:
            return

        self._wbs_lock.acquire()

        for wb in self._wb_to_remove:
            if "context" in wb.params:
                ctxt = wb.params['context']
                if ctxt:
                    ctxt["done"] = True
            self.watch_block_remove(wb)
        self._wb_to_remove = []

        self._wbs_lock.release()

    def _thread_dequeue_start(self):
        """
        Internal method to start dequeue thread.
        """

        self._dequeue_thread = Thread(name="dequeue_thread",
                                      target=self._dequeue_function)
        self._dequeue_thread.daemon = True
        self._dequeue_thread.start()

    def watch_block_add(self, params,
                        filter_callback,
                        capacity=500,
                        callback=None):
        """
        Internal method to add a watch block for starting event monitoring.
        """

        if self._handle.cookie is None:
            return None

        self._wbs_lock.acquire()
        watch_block = WatchBlock(params,
                                 filter_callback,
                                 capacity,
                                 callback)  # Add a List of Watchers

        if watch_block is not None and watch_block.callback is None:
            watch_block.callback = watch_block.dequeue_default_callback

        self._wbs.append(watch_block)
        self._wbs_lock.release()
        return watch_block

    def watch_block_remove(self, watch_block):
        """
        Internal method to remove a watch block for
        stopping event monitoring.
        """
        if watch_block in self._wbs:
            self._wbs.remove(watch_block)

    def _add_class_id_watch(self, class_id):
        if ucscoreutils.find_class_id_in_mo_meta_ignore_case(class_id) is None:
            raise UcsValidationException(
                "Invalid ClassId %s specified." % class_id)

        def watch__type_filter(mce):
            """
            Callback method to work on events with a specific class_id.
            """
            if mce.mo.get_class_id().lower() == class_id.lower():
                return True
            return False

        return watch__type_filter

    def _add_mo_watch(self, managed_object, prop=None, success_value=[],
                      poll_sec=None):
        if ucscoreutils.find_class_id_in_mo_meta_ignore_case(
                managed_object.get_class_id()) is None:
            raise UcsValidationException(
                "Unknown ClassId %s provided." %
                managed_object.get_class_id())

        if prop is not None:
            mo_property_meta = ucscoreutils.get_mo_property_meta(
                managed_object.get_class_id(), prop)
            if mo_property_meta is None:
                raise UcsValidationException(
                    "Unknown Property %s provided." % prop)

            if not success_value:
                raise UcsValidationException(
                    "success_value parameter is not provided.")

        if poll_sec is None:
            def watch_mo_filter(mce):
                """
                Callback method to work on events specific to respective
                managed object.
                """
                if mce.mo.dn == managed_object.dn:
                    return True
                return False
            return watch_mo_filter
        else:
            def watch_none_filter(mce):
                """
                Callback method to ignore all events.
                """
                return False
            return watch_none_filter

    def add(self,
            class_id=None,
            managed_object=None,
            prop=None,
            success_value=[],
            poll_sec=None,
            timeout_sec=None,
            call_back=None,
            context=None):
        """
        Adds an event handler.

        An event handler can be added using this method where an user can
        subscribe for the event channel from UCS and can monitor those events
        for any specific success value for a managed object.

        Args:
            class_id (str): managed object class id
            managed_object (ManagedObject)
            prop (str) - property of the managed object to monitor
            success_value (list) - success values of a prop
            poll_sec - specifies the time in seconds for polling event.
            timeout_sec - time after which method should stop monitoring.
            call_back - call back method
        """

        if class_id is not None and managed_object is not None:
            raise UcsValidationException(
                "Specify either class_id or managedObject, not both")

        if class_id is not None:
            filter_callback = self._add_class_id_watch(class_id)
        elif managed_object is not None:
            filter_callback = self._add_mo_watch(managed_object, prop,
                                                 success_value, poll_sec)
        else:
            def watch_all_filter(mce):
                """
                Callback method to work on all events.
                """
                return True
            filter_callback = watch_all_filter

        param_dict = {'class_id': class_id,
                      'managed_object': managed_object,
                      'prop': prop,
                      'success_value': success_value,
                      'poll_sec': poll_sec,
                      'timeout_sec': timeout_sec,
                      'call_back': call_back,
                      'start_time': datetime.datetime.now(),
                      'context': context}

        if filter_callback is None:
            raise UcsValidationException("Error adding WatchBlock...")

        watch_block = self.watch_block_add(params=param_dict,
                                           filter_callback=filter_callback,
                                           callback=call_back)

        if watch_block is not None and len(self._wbs) == 1:
            if poll_sec is None:
                self._thread_enqueue_start()
            self._thread_dequeue_start()

        return watch_block

    def remove(self, watch_block):
        """
        Removes an event handler.
        """

        self._wbs_lock.acquire()
        if watch_block in self._wbs:
            self.watch_block_remove(watch_block)
        else:
            UcsWarning("Event handler not found")
        self._wbs_lock.release()

    def clean(self):
        """
        Removes all the watch blocks from the event handler
        """

        self._wbs_lock.acquire()
        for each in self._wbs:
            self.watch_block_remove(each)
        self._wbs_lock.release()

    def get(self):
        """
        Returns the list of event handlers.
        """
        return self._wbs


def wait(handle, mo, prop, value, cb, timeout_sec=None, poll_sec=None):
    """
    Waits for `mo.prop == value`

    Args:
        handle(UcsHandle): connection handle to the server
        mo (Managed Object): managed object to watch
        prop (str): property to watch
        value (str): property value to wait for
        cb(function): callback on success
        timeout_sec (int): timeout
        poll_sec (int): polling interval in seconds

    Returns:
        None

    Example:
        This method is called from UcsHandle class,
        wait_for_event method
    """

    if mo is None:
        return

    # create a new event handler
    ueh = UcsEventHandle(handle)

    context = {}
    context["done"] = False

    if isinstance(value, list):
        success_value = value
    else:
        success_value = [value]

    # create a watch block
    ueh.add(
        managed_object=mo,
        prop=prop,
        success_value=success_value,
        call_back=cb,
        timeout_sec=timeout_sec,
        poll_sec=poll_sec,
        context=context)

    # wait for the event to occur
    while not context["done"]:
        time.sleep(1)



"""This module contains the meta information of ApeMcGetSmbios ExternalMethod."""

from ..ucscoremeta import MethodMeta, MethodPropertyMeta

method_meta = MethodMeta("ApeMcGetSmbios", "apeMcGetSmbios", "Version142b")

prop_meta = {
    "cookie": MethodPropertyMeta("Cookie", "cookie", "Xs:string", "Version142b", "InputOutput", False),
    "in_ip_addr": MethodPropertyMeta("InIpAddr", "inIpAddr", "AddressIPv4", "Version142b", "Input", False),
    "in_update_cnt": MethodPropertyMeta("InUpdateCnt", "inUpdateCnt", "Xs:int", "Version142b", "Input", False),
    "out_file_path": MethodPropertyMeta("OutFilePath", "outFilePath", "Xs:string", "Version142b", "Output", False),
    "out_update_cnt": MethodPropertyMeta("OutUpdateCnt", "outUpdateCnt", "Xs:int", "Version142b", "Output", False),
}

prop_map = {
    "cookie": "cookie",
    "inIpAddr": "in_ip_addr",
    "inUpdateCnt": "in_update_cnt",
    "outFilePath": "out_file_path",
    "outUpdateCnt": "out_update_cnt",
}



"""This module contains the general information for LsbootDef ManagedObject."""

from ...ucsmo import ManagedObject
from ...ucscoremeta import MoPropertyMeta, MoMeta
from ...ucsmeta import VersionMeta


class LsbootDefConsts:
    ADV_BOOT_ORDER_APPLICABLE_FALSE = "false"
    ADV_BOOT_ORDER_APPLICABLE_NO = "no"
    ADV_BOOT_ORDER_APPLICABLE_TRUE = "true"
    ADV_BOOT_ORDER_APPLICABLE_YES = "yes"
    BOOT_MODE_LEGACY = "legacy"
    BOOT_MODE_UEFI = "uefi"
    ENFORCE_VNIC_NAME_FALSE = "false"
    ENFORCE_VNIC_NAME_NO = "no"
    ENFORCE_VNIC_NAME_TRUE = "true"
    ENFORCE_VNIC_NAME_YES = "yes"
    INT_ID_NONE = "none"
    POLICY_OWNER_LOCAL = "local"
    POLICY_OWNER_PENDING_POLICY = "pending-policy"
    POLICY_OWNER_POLICY = "policy"
    PURPOSE_OPERATIONAL = "operational"
    PURPOSE_UTILITY = "utility"
    REBOOT_ON_UPDATE_FALSE = "false"
    REBOOT_ON_UPDATE_NO = "no"
    REBOOT_ON_UPDATE_TRUE = "true"
    REBOOT_ON_UPDATE_YES = "yes"


class LsbootDef(ManagedObject):
    """This is LsbootDef class."""

    consts = LsbootDefConsts()
    naming_props = set([])

    mo_meta = MoMeta("LsbootDef", "lsbootDef", "boot-policy", VersionMeta.Version101e, "InputOutput", 0x7ff, [], ["admin", "ls-compute", "ls-config", "ls-server", "ls-storage"], ['computeBlade', 'computeRackUnit', 'computeServerUnit', 'lsServer'], ['lsbootBootSecurity', 'lsbootEFIShell', 'lsbootIScsi', 'lsbootLan', 'lsbootSan', 'lsbootStorage', 'lsbootVirtualMedia'], ["Add", "Get", "Remove", "Set"])

    prop_meta = {
        "adv_boot_order_applicable": MoPropertyMeta("adv_boot_order_applicable", "advBootOrderApplicable", "string", VersionMeta.Version221b, MoPropertyMeta.READ_WRITE, 0x2, None, None, None, ["false", "no", "true", "yes"], []),
        "boot_mode": MoPropertyMeta("boot_mode", "bootMode", "string", VersionMeta.Version221b, MoPropertyMeta.READ_WRITE, 0x4, None, None, None, ["legacy", "uefi"], []),
        "child_action": MoPropertyMeta("child_action", "childAction", "string", VersionMeta.Version101e, MoPropertyMeta.INTERNAL, 0x8, None, None, r"""((deleteAll|ignore|deleteNonPresent),){0,2}(deleteAll|ignore|deleteNonPresent){0,1}""", [], []),
        "descr": MoPropertyMeta("descr", "descr", "string", VersionMeta.Version101e, MoPropertyMeta.READ_WRITE, 0x10, None, None, r"""[ !#$%&\(\)\*\+,\-\./:;\?@\[\]_\{\|\}~a-zA-Z0-9]{0,256}""", [], []),
        "dn": MoPropertyMeta("dn", "dn", "string", VersionMeta.Version101e, MoPropertyMeta.READ_ONLY, 0x20, 0, 256, None, [], []),
        "enforce_vnic_name": MoPropertyMeta("enforce_vnic_name", "enforceVnicName", "string", VersionMeta.Version102d, MoPropertyMeta.READ_WRITE, 0x40, None, None, None, ["false", "no", "true", "yes"], []),
        "int_id": MoPropertyMeta("int_id", "intId", "string", VersionMeta.Version101e, MoPropertyMeta.INTERNAL, None, None, None, None, ["none"], ["0-4294967295"]),
        "name": MoPropertyMeta("name", "name", "string", VersionMeta.Version101e, MoPropertyMeta.READ_ONLY, None, None, None, r"""[\-\.:_a-zA-Z0-9]{0,16}""", [], []),
        "policy_level": MoPropertyMeta("policy_level", "policyLevel", "uint", VersionMeta.Version211a, MoPropertyMeta.READ_ONLY, None, None, None, None, [], []),
        "policy_owner": MoPropertyMeta("policy_owner", "policyOwner", "string", VersionMeta.Version211a, MoPropertyMeta.READ_WRITE, 0x80, None, None, None, ["local", "pending-policy", "policy"], []),
        "purpose": MoPropertyMeta("purpose", "purpose", "string", VersionMeta.Version101e, MoPropertyMeta.READ_ONLY, None, None, None, None, ["operational", "utility"], []),
        "reboot_on_update": MoPropertyMeta("reboot_on_update", "rebootOnUpdate", "string", VersionMeta.Version101e, MoPropertyMeta.READ_WRITE, 0x100, None, None, None, ["false", "no", "true", "yes"], []),
        "rn": MoPropertyMeta("rn", "rn", "string", VersionMeta.Version101e, MoPropertyMeta.READ_ONLY, 0x200, 0, 256, None, [], []),
        "sacl": MoPropertyMeta("sacl", "sacl", "string", VersionMeta.Version302c, MoPropertyMeta.READ_ONLY, None, None, None, r"""((none|del|mod|addchild|cascade),){0,4}(none|del|mod|addchild|cascade){0,1}""", [], []),
        "status": MoPropertyMeta("status", "status", "string", VersionMeta.Version101e, MoPropertyMeta.READ_WRITE, 0x400, None, None, r"""((removed|created|modified|deleted),){0,3}(removed|created|modified|deleted){0,1}""", [], []),
    }

    prop_map = {
        "advBootOrderApplicable": "adv_boot_order_applicable", 
        "bootMode": "boot_mode", 
        "childAction": "child_action", 
        "descr": "descr", 
        "dn": "dn", 
        "enforceVnicName": "enforce_vnic_name", 
        "intId": "int_id", 
        "name": "name", 
        "policyLevel": "policy_level", 
        "policyOwner": "policy_owner", 
        "purpose": "purpose", 
        "rebootOnUpdate": "reboot_on_update", 
        "rn": "rn", 
        "sacl": "sacl", 
        "status": "status", 
    }

    def __init__(self, parent_mo_or_dn, **kwargs):
        self._dirty_mask = 0
        self.adv_boot_order_applicable = None
        self.boot_mode = None
        self.child_action = None
        self.descr = None
        self.enforce_vnic_name = None
        self.int_id = None
        self.name = None
        self.policy_level = None
        self.policy_owner = None
        self.purpose = None
        self.reboot_on_update = None
        self.sacl = None
        self.status = None

        ManagedObject.__init__(self, "LsbootDef", parent_mo_or_dn, **kwargs)



"""This module contains the general information for AaaSessionInfo ManagedObject."""

from ...ucsmo import ManagedObject
from ...ucscoremeta import MoPropertyMeta, MoMeta
from ...ucsmeta import VersionMeta


class AaaSessionInfoConsts:
    TYPE_ALL = "all"
    TYPE_KVM = "kvm"
    TYPE_SOL = "sol"
    TYPE_VMEDIA = "vmedia"
    USER_TYPE_IPMI = "ipmi"
    USER_TYPE_LOCAL = "local"
    USER_TYPE_REMOTE = "remote"


class AaaSessionInfo(ManagedObject):
    """This is AaaSessionInfo class."""

    consts = AaaSessionInfoConsts()
    naming_props = set(['id'])

    mo_meta = MoMeta("AaaSessionInfo", "aaaSessionInfo", "term-[id]", VersionMeta.Version212a, "InputOutput", 0x1fff, [], ["aaa", "admin"], ['aaaSessionInfoTable'], [], [None])

    prop_meta = {
        "address": MoPropertyMeta("address", "address", "string", VersionMeta.Version212a, MoPropertyMeta.READ_WRITE, 0x2, 0, 510, None, [], []),
        "child_action": MoPropertyMeta("child_action", "childAction", "string", VersionMeta.Version212a, MoPropertyMeta.INTERNAL, 0x4, None, None, r"""((deleteAll|ignore|deleteNonPresent),){0,2}(deleteAll|ignore|deleteNonPresent){0,1}""", [], []),
        "dest_ip": MoPropertyMeta("dest_ip", "destIp", "string", VersionMeta.Version212a, MoPropertyMeta.READ_WRITE, 0x8, 0, 510, None, [], []),
        "dn": MoPropertyMeta("dn", "dn", "string", VersionMeta.Version212a, MoPropertyMeta.READ_ONLY, 0x10, 0, 256, None, [], []),
        "etime": MoPropertyMeta("etime", "etime", "string", VersionMeta.Version212a, MoPropertyMeta.READ_WRITE, 0x20, None, None, r"""([0-9]){4}-([0-9]){2}-([0-9]){2}T([0-9]){2}:([0-9]){2}:([0-9]){2}((\.([0-9]){3})){0,1}""", [], []),
        "id": MoPropertyMeta("id", "id", "string", VersionMeta.Version212a, MoPropertyMeta.NAMING, 0x40, 1, 510, None, [], []),
        "priv": MoPropertyMeta("priv", "priv", "string", VersionMeta.Version212a, MoPropertyMeta.READ_WRITE, 0x80, 0, 510, None, [], []),
        "rn": MoPropertyMeta("rn", "rn", "string", VersionMeta.Version212a, MoPropertyMeta.READ_ONLY, 0x100, 0, 256, None, [], []),
        "sacl": MoPropertyMeta("sacl", "sacl", "string", VersionMeta.Version302c, MoPropertyMeta.READ_ONLY, None, None, None, r"""((none|del|mod|addchild|cascade),){0,4}(none|del|mod|addchild|cascade){0,1}""", [], []),
        "status": MoPropertyMeta("status", "status", "string", VersionMeta.Version212a, MoPropertyMeta.READ_WRITE, 0x200, None, None, r"""((removed|created|modified|deleted),){0,3}(removed|created|modified|deleted){0,1}""", [], []),
        "type": MoPropertyMeta("type", "type", "string", VersionMeta.Version212a, MoPropertyMeta.READ_WRITE, 0x400, None, None, None, ["all", "kvm", "sol", "vmedia"], []),
        "user": MoPropertyMeta("user", "user", "string", VersionMeta.Version212a, MoPropertyMeta.READ_WRITE, 0x800, 0, 510, None, [], []),
        "user_type": MoPropertyMeta("user_type", "userType", "string", VersionMeta.Version212a, MoPropertyMeta.READ_WRITE, 0x1000, None, None, None, ["ipmi", "local", "remote"], []),
    }

    prop_map = {
        "address": "address", 
        "childAction": "child_action", 
        "destIp": "dest_ip", 
        "dn": "dn", 
        "etime": "etime", 
        "id": "id", 
        "priv": "priv", 
        "rn": "rn", 
        "sacl": "sacl", 
        "status": "status", 
        "type": "type", 
        "user": "user", 
        "userType": "user_type", 
    }

    def __init__(self, parent_mo_or_dn, id, **kwargs):
        self._dirty_mask = 0
        self.id = id
        self.address = None
        self.child_action = None
        self.dest_ip = None
        self.etime = None
        self.priv = None
        self.sacl = None
        self.status = None
        self.type = None
        self.user = None
        self.user_type = None

        ManagedObject.__init__(self, "AaaSessionInfo", parent_mo_or_dn, **kwargs)



"""This module contains the meta information of ApeGetServerFromIp ExternalMethod."""

from ..ucscoremeta import MethodMeta, MethodPropertyMeta

method_meta = MethodMeta("ApeGetServerFromIp", "apeGetServerFromIp", "Version142b")

prop_meta = {
    "cookie": MethodPropertyMeta("Cookie", "cookie", "Xs:string", "Version142b", "InputOutput", False),
    "in_ip_addr": MethodPropertyMeta("InIpAddr", "inIpAddr", "AddressIPv4", "Version142b", "Input", False),
    "out_chassis_id": MethodPropertyMeta("OutChassisId", "outChassisId", "Xs:string", "Version142b", "Output", False),
    "out_server_instance_id": MethodPropertyMeta("OutServerInstanceId", "outServerInstanceId", "Xs:unsignedInt", "Version142b", "Output", False),
    "out_slot_id": MethodPropertyMeta("OutSlotId", "outSlotId", "Xs:unsignedInt", "Version142b", "Output", False),
}

prop_map = {
    "cookie": "cookie",
    "inIpAddr": "in_ip_addr",
    "outChassisId": "out_chassis_id",
    "outServerInstanceId": "out_server_instance_id",
    "outSlotId": "out_slot_id",
}



"""This module contains the general information for AdaptorFcIfFC4Stats ManagedObject."""

from ...ucsmo import ManagedObject
from ...ucscoremeta import MoPropertyMeta, MoMeta
from ...ucsmeta import VersionMeta


class AdaptorFcIfFC4StatsConsts:
    CONTROL_REQUESTS_NA = "NA"
    CONTROL_REQUESTS_DELTA_NA = "NA"
    CONTROL_REQUESTS_DELTA_AVG_NA = "NA"
    CONTROL_REQUESTS_DELTA_MAX_NA = "NA"
    CONTROL_REQUESTS_DELTA_MIN_NA = "NA"
    INPUT_MEGABYTES_NA = "NA"
    INPUT_MEGABYTES_DELTA_NA = "NA"
    INPUT_MEGABYTES_DELTA_AVG_NA = "NA"
    INPUT_MEGABYTES_DELTA_MAX_NA = "NA"
    INPUT_MEGABYTES_DELTA_MIN_NA = "NA"
    INPUT_REQUESTS_NA = "NA"
    INPUT_REQUESTS_DELTA_NA = "NA"
    INPUT_REQUESTS_DELTA_AVG_NA = "NA"
    INPUT_REQUESTS_DELTA_MAX_NA = "NA"
    INPUT_REQUESTS_DELTA_MIN_NA = "NA"
    OUTPUT_MEGABYTES_NA = "NA"
    OUTPUT_MEGABYTES_DELTA_NA = "NA"
    OUTPUT_MEGABYTES_DELTA_AVG_NA = "NA"
    OUTPUT_MEGABYTES_DELTA_MAX_NA = "NA"
    OUTPUT_MEGABYTES_DELTA_MIN_NA = "NA"
    OUTPUT_REQUESTS_NA = "NA"
    OUTPUT_REQUESTS_DELTA_NA = "NA"
    OUTPUT_REQUESTS_DELTA_AVG_NA = "NA"
    OUTPUT_REQUESTS_DELTA_MAX_NA = "NA"
    OUTPUT_REQUESTS_DELTA_MIN_NA = "NA"
    SUSPECT_FALSE = "false"
    SUSPECT_NO = "no"
    SUSPECT_TRUE = "true"
    SUSPECT_YES = "yes"


class AdaptorFcIfFC4Stats(ManagedObject):
    """This is AdaptorFcIfFC4Stats class."""

    consts = AdaptorFcIfFC4StatsConsts()
    naming_props = set([])

    mo_meta = MoMeta("AdaptorFcIfFC4Stats", "adaptorFcIfFC4Stats", "fc-if-fc4-stats", VersionMeta.Version111j, "OutputOnly", 0xf, [], ["admin", "operations", "read-only"], ['adaptorHostFcIf'], ['adaptorFcIfFC4StatsHist'], ["Get"])

    prop_meta = {
        "child_action": MoPropertyMeta("child_action", "childAction", "string", VersionMeta.Version111j, MoPropertyMeta.INTERNAL, None, None, None, r"""((deleteAll|ignore|deleteNonPresent),){0,2}(deleteAll|ignore|deleteNonPresent){0,1}""", [], []),
        "control_requests": MoPropertyMeta("control_requests", "controlRequests", "string", VersionMeta.Version111j, MoPropertyMeta.READ_ONLY, None, None, None, None, ["NA"], ["0-18446744073709551615"]),
        "control_requests_delta": MoPropertyMeta("control_requests_delta", "controlRequestsDelta", "string", VersionMeta.Version111j, MoPropertyMeta.READ_ONLY, None, None, None, None, ["NA"], ["0-18446744073709551615"]),
        "control_requests_delta_avg": MoPropertyMeta("control_requests_delta_avg", "controlRequestsDeltaAvg", "string", VersionMeta.Version111j, MoPropertyMeta.READ_ONLY, None, None, None, None, ["NA"], ["0-18446744073709551615"]),
        "control_requests_delta_max": MoPropertyMeta("control_requests_delta_max", "controlRequestsDeltaMax", "string", VersionMeta.Version111j, MoPropertyMeta.READ_ONLY, None, None, None, None, ["NA"], ["0-18446744073709551615"]),
        "control_requests_delta_min": MoPropertyMeta("control_requests_delta_min", "controlRequestsDeltaMin", "string", VersionMeta.Version111j, MoPropertyMeta.READ_ONLY, None, None, None, None, ["NA"], ["0-18446744073709551615"]),
        "dn": MoPropertyMeta("dn", "dn", "string", VersionMeta.Version111j, MoPropertyMeta.READ_ONLY, 0x2, 0, 256, None, [], []),
        "input_megabytes": MoPropertyMeta("input_megabytes", "inputMegabytes", "string", VersionMeta.Version111j, MoPropertyMeta.READ_ONLY, None, None, None, None, ["NA"], ["0-18446744073709551615"]),
        "input_megabytes_delta": MoPropertyMeta("input_megabytes_delta", "inputMegabytesDelta", "string", VersionMeta.Version111j, MoPropertyMeta.READ_ONLY, None, None, None, None, ["NA"], ["0-18446744073709551615"]),
        "input_megabytes_delta_avg": MoPropertyMeta("input_megabytes_delta_avg", "inputMegabytesDeltaAvg", "string", VersionMeta.Version111j, MoPropertyMeta.READ_ONLY, None, None, None, None, ["NA"], ["0-18446744073709551615"]),
        "input_megabytes_delta_max": MoPropertyMeta("input_megabytes_delta_max", "inputMegabytesDeltaMax", "string", VersionMeta.Version111j, MoPropertyMeta.READ_ONLY, None, None, None, None, ["NA"], ["0-18446744073709551615"]),
        "input_megabytes_delta_min": MoPropertyMeta("input_megabytes_delta_min", "inputMegabytesDeltaMin", "string", VersionMeta.Version111j, MoPropertyMeta.READ_ONLY, None, None, None, None, ["NA"], ["0-18446744073709551615"]),
        "input_requests": MoPropertyMeta("input_requests", "inputRequests", "string", VersionMeta.Version111j, MoPropertyMeta.READ_ONLY, None, None, None, None, ["NA"], ["0-18446744073709551615"]),
        "input_requests_delta": MoPropertyMeta("input_requests_delta", "inputRequestsDelta", "string", VersionMeta.Version111j, MoPropertyMeta.READ_ONLY, None, None, None, None, ["NA"], ["0-18446744073709551615"]),
        "input_requests_delta_avg": MoPropertyMeta("input_requests_delta_avg", "inputRequestsDeltaAvg", "string", VersionMeta.Version111j, MoPropertyMeta.READ_ONLY, None, None, None, None, ["NA"], ["0-18446744073709551615"]),
        "input_requests_delta_max": MoPropertyMeta("input_requests_delta_max", "inputRequestsDeltaMax", "string", VersionMeta.Version111j, MoPropertyMeta.READ_ONLY, None, None, None, None, ["NA"], ["0-18446744073709551615"]),
        "input_requests_delta_min": MoPropertyMeta("input_requests_delta_min", "inputRequestsDeltaMin", "string", VersionMeta.Version111j, MoPropertyMeta.READ_ONLY, None, None, None, None, ["NA"], ["0-18446744073709551615"]),
        "intervals": MoPropertyMeta("intervals", "intervals", "uint", VersionMeta.Version111j, MoPropertyMeta.READ_ONLY, None, None, None, None, [], []),
        "output_megabytes": MoPropertyMeta("output_megabytes", "outputMegabytes", "string", VersionMeta.Version111j, MoPropertyMeta.READ_ONLY, None, None, None, None, ["NA"], ["0-18446744073709551615"]),
        "output_megabytes_delta": MoPropertyMeta("output_megabytes_delta", "outputMegabytesDelta", "string", VersionMeta.Version111j, MoPropertyMeta.READ_ONLY, None, None, None, None, ["NA"], ["0-18446744073709551615"]),
        "output_megabytes_delta_avg": MoPropertyMeta("output_megabytes_delta_avg", "outputMegabytesDeltaAvg", "string", VersionMeta.Version111j, MoPropertyMeta.READ_ONLY, None, None, None, None, ["NA"], ["0-18446744073709551615"]),
        "output_megabytes_delta_max": MoPropertyMeta("output_megabytes_delta_max", "outputMegabytesDeltaMax", "string", VersionMeta.Version111j, MoPropertyMeta.READ_ONLY, None, None, None, None, ["NA"], ["0-18446744073709551615"]),
        "output_megabytes_delta_min": MoPropertyMeta("output_megabytes_delta_min", "outputMegabytesDeltaMin", "string", VersionMeta.Version111j, MoPropertyMeta.READ_ONLY, None, None, None, None, ["NA"], ["0-18446744073709551615"]),
        "output_requests": MoPropertyMeta("output_requests", "outputRequests", "string", VersionMeta.Version111j, MoPropertyMeta.READ_ONLY, None, None, None, None, ["NA"], ["0-18446744073709551615"]),
        "output_requests_delta": MoPropertyMeta("output_requests_delta", "outputRequestsDelta", "string", VersionMeta.Version111j, MoPropertyMeta.READ_ONLY, None, None, None, None, ["NA"], ["0-18446744073709551615"]),
        "output_requests_delta_avg": MoPropertyMeta("output_requests_delta_avg", "outputRequestsDeltaAvg", "string", VersionMeta.Version111j, MoPropertyMeta.READ_ONLY, None, None, None, None, ["NA"], ["0-18446744073709551615"]),
        "output_requests_delta_max": MoPropertyMeta("output_requests_delta_max", "outputRequestsDeltaMax", "string", VersionMeta.Version111j, MoPropertyMeta.READ_ONLY, None, None, None, None, ["NA"], ["0-18446744073709551615"]),
        "output_requests_delta_min": MoPropertyMeta("output_requests_delta_min", "outputRequestsDeltaMin", "string", VersionMeta.Version111j, MoPropertyMeta.READ_ONLY, None, None, None, None, ["NA"], ["0-18446744073709551615"]),
        "rn": MoPropertyMeta("rn", "rn", "string", VersionMeta.Version111j, MoPropertyMeta.READ_ONLY, 0x4, 0, 256, None, [], []),
        "sacl": MoPropertyMeta("sacl", "sacl", "string", VersionMeta.Version302c, MoPropertyMeta.READ_ONLY, None, None, None, r"""((none|del|mod|addchild|cascade),){0,4}(none|del|mod|addchild|cascade){0,1}""", [], []),
        "status": MoPropertyMeta("status", "status", "string", VersionMeta.Version111j, MoPropertyMeta.READ_WRITE, 0x8, None, None, r"""((removed|created|modified|deleted),){0,3}(removed|created|modified|deleted){0,1}""", [], []),
        "suspect": MoPropertyMeta("suspect", "suspect", "string", VersionMeta.Version111j, MoPropertyMeta.READ_ONLY, None, None, None, None, ["false", "no", "true", "yes"], []),
        "thresholded": MoPropertyMeta("thresholded", "thresholded", "string", VersionMeta.Version111j, MoPropertyMeta.READ_ONLY, None, None, None, None, [], []),
        "time_collected": MoPropertyMeta("time_collected", "timeCollected", "string", VersionMeta.Version111j, MoPropertyMeta.READ_ONLY, None, None, None, r"""([0-9]){4}-([0-9]){2}-([0-9]){2}T([0-9]){2}:([0-9]){2}:([0-9]){2}((\.([0-9]){3})){0,1}""", [], []),
        "update": MoPropertyMeta("update", "update", "uint", VersionMeta.Version111j, MoPropertyMeta.READ_ONLY, None, None, None, None, [], []),
    }

    prop_map = {
        "childAction": "child_action", 
        "controlRequests": "control_requests", 
        "controlRequestsDelta": "control_requests_delta", 
        "controlRequestsDeltaAvg": "control_requests_delta_avg", 
        "controlRequestsDeltaMax": "control_requests_delta_max", 
        "controlRequestsDeltaMin": "control_requests_delta_min", 
        "dn": "dn", 
        "inputMegabytes": "input_megabytes", 
        "inputMegabytesDelta": "input_megabytes_delta", 
        "inputMegabytesDeltaAvg": "input_megabytes_delta_avg", 
        "inputMegabytesDeltaMax": "input_megabytes_delta_max", 
        "inputMegabytesDeltaMin": "input_megabytes_delta_min", 
        "inputRequests": "input_requests", 
        "inputRequestsDelta": "input_requests_delta", 
        "inputRequestsDeltaAvg": "input_requests_delta_avg", 
        "inputRequestsDeltaMax": "input_requests_delta_max", 
        "inputRequestsDeltaMin": "input_requests_delta_min", 
        "intervals": "intervals", 
        "outputMegabytes": "output_megabytes", 
        "outputMegabytesDelta": "output_megabytes_delta", 
        "outputMegabytesDeltaAvg": "output_megabytes_delta_avg", 
        "outputMegabytesDeltaMax": "output_megabytes_delta_max", 
        "outputMegabytesDeltaMin": "output_megabytes_delta_min", 
        "outputRequests": "output_requests", 
        "outputRequestsDelta": "output_requests_delta", 
        "outputRequestsDeltaAvg": "output_requests_delta_avg", 
        "outputRequestsDeltaMax": "output_requests_delta_max", 
        "outputRequestsDeltaMin": "output_requests_delta_min", 
        "rn": "rn", 
        "sacl": "sacl", 
        "status": "status", 
        "suspect": "suspect", 
        "thresholded": "thresholded", 
        "timeCollected": "time_collected", 
        "update": "update", 
    }

    def __init__(self, parent_mo_or_dn, **kwargs):
        self._dirty_mask = 0
        self.child_action = None
        self.control_requests = None
        self.control_requests_delta = None
        self.control_requests_delta_avg = None
        self.control_requests_delta_max = None
        self.control_requests_delta_min = None
        self.input_megabytes = None
        self.input_megabytes_delta = None
        self.input_megabytes_delta_avg = None
        self.input_megabytes_delta_max = None
        self.input_megabytes_delta_min = None
        self.input_requests = None
        self.input_requests_delta = None
        self.input_requests_delta_avg = None
        self.input_requests_delta_max = None
        self.input_requests_delta_min = None
        self.intervals = None
        self.output_megabytes = None
        self.output_megabytes_delta = None
        self.output_megabytes_delta_avg = None
        self.output_megabytes_delta_max = None
        self.output_megabytes_delta_min = None
        self.output_requests = None
        self.output_requests_delta = None
        self.output_requests_delta_avg = None
        self.output_requests_delta_max = None
        self.output_requests_delta_min = None
        self.sacl = None
        self.status = None
        self.suspect = None
        self.thresholded = None
        self.time_collected = None
        self.update = None

        ManagedObject.__init__(self, "AdaptorFcIfFC4Stats", parent_mo_or_dn, **kwargs)



"""This module contains the meta information of AaaGetKVMLaunchUrl ExternalMethod."""

from ..ucscoremeta import MethodMeta, MethodPropertyMeta

method_meta = MethodMeta("AaaGetKVMLaunchUrl", "aaaGetKVMLaunchUrl", "Version142b")

prop_meta = {
    "cookie": MethodPropertyMeta("Cookie", "cookie", "Xs:string", "Version142b", "InputOutput", False),
    "in_cookie": MethodPropertyMeta("InCookie", "inCookie", "Xs:string", "Version142b", "Input", False),
    "in_ipv4": MethodPropertyMeta("InIpv4", "inIpv4", "Xs:string", "Version142b", "Input", False),
    "out_url": MethodPropertyMeta("OutUrl", "outUrl", "Xs:string", "Version142b", "Output", False),
}

prop_map = {
    "cookie": "cookie",
    "inCookie": "in_cookie",
    "inIpv4": "in_ipv4",
    "outUrl": "out_url",
}



"""This module contains the general information for StorageVirtualDrive ManagedObject."""

from ...ucsmo import ManagedObject
from ...ucscoremeta import MoPropertyMeta, MoMeta
from ...ucsmeta import VersionMeta


class StorageVirtualDriveConsts:
    ACCESS_POLICY_BLOCKED = "blocked"
    ACCESS_POLICY_HIDDEN = "hidden"
    ACCESS_POLICY_READ_ONLY = "read-only"
    ACCESS_POLICY_READ_WRITE = "read-write"
    ACCESS_POLICY_TRANSPORT_READY = "transport-ready"
    ACCESS_POLICY_UNKNOWN = "unknown"
    ACTUAL_WRITE_CACHE_POLICY_UNKNOWN = "unknown"
    ACTUAL_WRITE_CACHE_POLICY_WRITE_BACK = "write-back"
    ACTUAL_WRITE_CACHE_POLICY_WRITE_THROUGH = "write-through"
    ADMIN_ACTION_TRIGGER_CANCELED = "canceled"
    ADMIN_ACTION_TRIGGER_IDLE = "idle"
    ADMIN_ACTION_TRIGGER_TRIGGERED = "triggered"
    ADMIN_STATE_CLEAR_TRANSPORT_READY = "clear-transport-ready"
    ADMIN_STATE_DEGRADED = "degraded"
    ADMIN_STATE_DELETE = "delete"
    ADMIN_STATE_HIDE = "hide"
    ADMIN_STATE_OFFLINE = "offline"
    ADMIN_STATE_ONLINE = "online"
    ADMIN_STATE_RESTORE = "restore"
    ADMIN_STATE_SECURE_DRIVE_GROUP = "secure-drive-group"
    ADMIN_STATE_TRANSPORT_READY = "transport-ready"
    ADMIN_STATE_UNDEFINED = "undefined"
    ADMIN_STATE_UNHIDE = "unhide"
    AVAILABLE_SIZE_UNKNOWN = "unknown"
    BLOCK_SIZE_512 = "512"
    BLOCK_SIZE_UNKNOWN = "unknown"
    BOOTABLE_FALSE = "false"
    BOOTABLE_TRUE = "true"
    BOOTABLE_UNKNOWN = "unknown"
    CONFIG_STATE_N_A = "N/A"
    CONFIG_STATE_APPLIED = "applied"
    CONFIG_STATE_APPLY_FAILED = "apply-failed"
    CONFIG_STATE_APPLYING = "applying"
    CONFIG_STATE_NOT_APPLIED = "not-applied"
    CONFIG_STATE_NOT_IN_USE = "not-in-use"
    CONFIG_STATE_ORPHANED = "orphaned"
    CONFIG_STATE_UNKNOWN = "unknown"
    CONFIGURED_WRITE_CACHE_POLICY_ALWAYS_WRITE_BACK = "always-write-back"
    CONFIGURED_WRITE_CACHE_POLICY_UNKNOWN = "unknown"
    CONFIGURED_WRITE_CACHE_POLICY_WRITE_BACK_GOOD_BBU = "write-back-good-bbu"
    CONFIGURED_WRITE_CACHE_POLICY_WRITE_THROUGH = "write-through"
    CONNECTION_PROTOCOL_NVME = "NVME"
    CONNECTION_PROTOCOL_SAS = "SAS"
    CONNECTION_PROTOCOL_SATA = "SATA"
    CONNECTION_PROTOCOL_UNSPECIFIED = "unspecified"
    DEPLOY_ACTION_ABORT_REPLICATION = "abort-replication"
    DEPLOY_ACTION_CREATE = "create"
    DEPLOY_ACTION_DELETE = "delete"
    DEPLOY_ACTION_MODIFY = "modify"
    DEPLOY_ACTION_NO_ACTION = "no-action"
    DEPLOY_ACTION_REPLACE = "replace"
    DEPLOY_ACTION_RESTORE = "restore"
    DEPLOY_ACTION_SET_OFFLINE = "set-offline"
    DEPLOY_ACTION_SET_ONLINE = "set-online"
    DRIVE_CACHE_DISABLE = "disable"
    DRIVE_CACHE_ENABLE = "enable"
    DRIVE_CACHE_NO_CHANGE = "no-change"
    DRIVE_CACHE_UNKNOWN = "unknown"
    DRIVE_SECURITY_FALSE = "false"
    DRIVE_SECURITY_NO = "no"
    DRIVE_SECURITY_TRUE = "true"
    DRIVE_SECURITY_YES = "yes"
    DRIVE_STATE_CACHE_DEGRADED = "cache-degraded"
    DRIVE_STATE_DEGRADED = "degraded"
    DRIVE_STATE_OFFLINE = "offline"
    DRIVE_STATE_OPTIMAL = "optimal"
    DRIVE_STATE_PARTIALLY_DEGRADED = "partially-degraded"
    DRIVE_STATE_REBUILDING = "rebuilding"
    DRIVE_STATE_UNKNOWN = "unknown"
    ID_UNSPECIFIED = "unspecified"
    IO_POLICY_CACHED = "cached"
    IO_POLICY_DIRECT = "direct"
    IO_POLICY_UNKNOWN = "unknown"
    LC_ALLOCATED = "allocated"
    LC_AVAILABLE = "available"
    LC_DEALLOCATED = "deallocated"
    LC_REPURPOSED = "repurposed"
    NUMBER_OF_BLOCKS_UNKNOWN = "unknown"
    OPER_DEVICE_ID_UNSPECIFIED = "unspecified"
    OPER_STATE_COMPUTE_DEGRADED = "compute-degraded"
    OPER_STATE_COMPUTE_INOPERABLE = "compute-inoperable"
    OPER_STATE_OFFLINE = "offline"
    OPER_STATE_ONLINE = "online"
    OPER_STATE_UNDEFINED = "undefined"
    OPERABILITY_ACCESSIBILITY_PROBLEM = "accessibility-problem"
    OPERABILITY_AUTO_UPGRADE = "auto-upgrade"
    OPERABILITY_BACKPLANE_PORT_PROBLEM = "backplane-port-problem"
    OPERABILITY_BIOS_POST_TIMEOUT = "bios-post-timeout"
    OPERABILITY_CHASSIS_INTRUSION = "chassis-intrusion"
    OPERABILITY_CHASSIS_LIMIT_EXCEEDED = "chassis-limit-exceeded"
    OPERABILITY_CONFIG = "config"
    OPERABILITY_DECOMISSIONING = "decomissioning"
    OPERABILITY_DEGRADED = "degraded"
    OPERABILITY_DISABLED = "disabled"
    OPERABILITY_DISCOVERY = "discovery"
    OPERABILITY_DISCOVERY_FAILED = "discovery-failed"
    OPERABILITY_EQUIPMENT_PROBLEM = "equipment-problem"
    OPERABILITY_FABRIC_CONN_PROBLEM = "fabric-conn-problem"
    OPERABILITY_FABRIC_UNSUPPORTED_CONN = "fabric-unsupported-conn"
    OPERABILITY_IDENTIFY = "identify"
    OPERABILITY_IDENTITY_UNESTABLISHABLE = "identity-unestablishable"
    OPERABILITY_INOPERABLE = "inoperable"
    OPERABILITY_LINK_ACTIVATE_BLOCKED = "link-activate-blocked"
    OPERABILITY_MALFORMED_FRU = "malformed-fru"
    OPERABILITY_NON_OPTIMAL = "non-optimal"
    OPERABILITY_NON_OPTIMAL_SEVERE = "non-optimal-severe"
    OPERABILITY_NOT_SUPPORTED = "not-supported"
    OPERABILITY_OPERABLE = "operable"
    OPERABILITY_PEER_COMM_PROBLEM = "peer-comm-problem"
    OPERABILITY_PERFORMANCE_PROBLEM = "performance-problem"
    OPERABILITY_POST_FAILURE = "post-failure"
    OPERABILITY_POWER_PROBLEM = "power-problem"
    OPERABILITY_POWERED_OFF = "powered-off"
    OPERABILITY_REMOVED = "removed"
    OPERABILITY_THERMAL_PROBLEM = "thermal-problem"
    OPERABILITY_UNKNOWN = "unknown"
    OPERABILITY_UNSUPPORTED_CONFIG = "unsupported-config"
    OPERABILITY_UPGRADE_PROBLEM = "upgrade-problem"
    OPERABILITY_VOLTAGE_PROBLEM = "voltage-problem"
    PHYSICAL_BLOCK_SIZE_512 = "512"
    PHYSICAL_BLOCK_SIZE_UNKNOWN = "unknown"
    PRESENCE_EMPTY = "empty"
    PRESENCE_EQUIPPED = "equipped"
    PRESENCE_EQUIPPED_DEPRECATED = "equipped-deprecated"
    PRESENCE_EQUIPPED_DISC_ERROR = "equipped-disc-error"
    PRESENCE_EQUIPPED_DISC_IN_PROGRESS = "equipped-disc-in-progress"
    PRESENCE_EQUIPPED_DISC_NOT_STARTED = "equipped-disc-not-started"
    PRESENCE_EQUIPPED_DISC_UNKNOWN = "equipped-disc-unknown"
    PRESENCE_EQUIPPED_IDENTITY_UNESTABLISHABLE = "equipped-identity-unestablishable"
    PRESENCE_EQUIPPED_NOT_PRIMARY = "equipped-not-primary"
    PRESENCE_EQUIPPED_SLAVE = "equipped-slave"
    PRESENCE_EQUIPPED_UNSUPPORTED = "equipped-unsupported"
    PRESENCE_EQUIPPED_WITH_MALFORMED_FRU = "equipped-with-malformed-fru"
    PRESENCE_INACCESSIBLE = "inaccessible"
    PRESENCE_MISMATCH = "mismatch"
    PRESENCE_MISMATCH_IDENTITY_UNESTABLISHABLE = "mismatch-identity-unestablishable"
    PRESENCE_MISMATCH_SLAVE = "mismatch-slave"
    PRESENCE_MISSING = "missing"
    PRESENCE_MISSING_SLAVE = "missing-slave"
    PRESENCE_NOT_SUPPORTED = "not-supported"
    PRESENCE_UNAUTHORIZED = "unauthorized"
    PRESENCE_UNKNOWN = "unknown"
    READ_POLICY_NORMAL = "normal"
    READ_POLICY_READ_AHEAD = "read-ahead"
    READ_POLICY_UNKNOWN = "unknown"
    SIZE_NOT_APPLICABLE = "not-applicable"
    STRIP_SIZE_UNKNOWN = "unknown"
    TYPE_MIRROR = "mirror"
    TYPE_MIRROR_STRIPE = "mirror-stripe"
    TYPE_RAID = "raid"
    TYPE_SIMPLE = "simple"
    TYPE_STRIPE = "stripe"
    TYPE_STRIPE_DUAL_PARITY = "stripe-dual-parity"
    TYPE_STRIPE_DUAL_PARITY_STRIPE = "stripe-dual-parity-stripe"
    TYPE_STRIPE_PARITY = "stripe-parity"
    TYPE_STRIPE_PARITY_STRIPE = "stripe-parity-stripe"
    TYPE_UNSPECIFIED = "unspecified"


class StorageVirtualDrive(ManagedObject):
    """This is StorageVirtualDrive class."""

    consts = StorageVirtualDriveConsts()
    naming_props = set(['id'])

    mo_meta = MoMeta("StorageVirtualDrive", "storageVirtualDrive", "vd-[id]", VersionMeta.Version211a, "InputOutput", 0x3ff, [], ["admin", "ls-compute", "ls-config", "ls-server", "ls-storage"], ['storageController', 'storageVirtualDriveContainer'], ['faultInst', 'storageControllerEp', 'storageLunDisk', 'storageOperation', 'storageScsiLunRef', 'storageVDMemberEp'], [None])

    prop_meta = {
        "access_policy": MoPropertyMeta("access_policy", "accessPolicy", "string", VersionMeta.Version221b, MoPropertyMeta.READ_ONLY, None, None, None, None, ["blocked", "hidden", "read-only", "read-write", "transport-ready", "unknown"], []),
        "actual_write_cache_policy": MoPropertyMeta("actual_write_cache_policy", "actualWriteCachePolicy", "string", VersionMeta.Version221b, MoPropertyMeta.READ_ONLY, None, None, None, None, ["unknown", "write-back", "write-through"], []),
        "admin_action_trigger": MoPropertyMeta("admin_action_trigger", "adminActionTrigger", "string", VersionMeta.Version224b, MoPropertyMeta.READ_WRITE, 0x2, None, None, None, ["canceled", "idle", "triggered"], []),
        "admin_name": MoPropertyMeta("admin_name", "adminName", "string", VersionMeta.Version224b, MoPropertyMeta.READ_WRITE, 0x4, None, None, r"""[\-\.:_a-zA-Z0-9]{0,15}""", [], []),
        "admin_state": MoPropertyMeta("admin_state", "adminState", "string", VersionMeta.Version224b, MoPropertyMeta.READ_WRITE, 0x8, None, None, None, ["clear-transport-ready", "degraded", "delete", "hide", "offline", "online", "restore", "secure-drive-group", "transport-ready", "undefined", "unhide"], []),
        "available_size": MoPropertyMeta("available_size", "availableSize", "string", VersionMeta.Version224b, MoPropertyMeta.READ_ONLY, None, None, None, None, ["unknown"], ["0-18446744073709551615"]),
        "block_size": MoPropertyMeta("block_size", "blockSize", "string", VersionMeta.Version211a, MoPropertyMeta.READ_ONLY, None, None, None, None, ["512", "unknown"], ["0-4294967295"]),
        "bootable": MoPropertyMeta("bootable", "bootable", "string", VersionMeta.Version221b, MoPropertyMeta.READ_ONLY, None, None, None, None, ["false", "true", "unknown"], []),
        "change_id": MoPropertyMeta("change_id", "changeId", "ulong", VersionMeta.Version227b, MoPropertyMeta.READ_ONLY, None, None, None, None, [], ["0-18446744073709551615"]),
        "change_qualifier": MoPropertyMeta("change_qualifier", "changeQualifier", "string", VersionMeta.Version224b, MoPropertyMeta.READ_ONLY, None, None, None, r"""((defaultValue|no-change|policy-change|global-hotspare-change|ded-hotspare-change|name-change|size-change|boot-drive-change|scrub-change),){0,8}(defaultValue|no-change|policy-change|global-hotspare-change|ded-hotspare-change|name-change|size-change|boot-drive-change|scrub-change){0,1}""", [], []),
        "child_action": MoPropertyMeta("child_action", "childAction", "string", VersionMeta.Version211a, MoPropertyMeta.INTERNAL, 0x10, None, None, r"""((deleteAll|ignore|deleteNonPresent),){0,2}(deleteAll|ignore|deleteNonPresent){0,1}""", [], []),
        "config_qualifier_reason": MoPropertyMeta("config_qualifier_reason", "configQualifierReason", "string", VersionMeta.Version224b, MoPropertyMeta.READ_ONLY, None, None, None, r"""[ !#$%&\(\)\*\+,\-\./:;\?@\[\]_\{\|\}~a-zA-Z0-9]{0,256}""", [], []),
        "config_state": MoPropertyMeta("config_state", "configState", "string", VersionMeta.Version224b, MoPropertyMeta.READ_ONLY, None, None, None, None, ["N/A", "applied", "apply-failed", "applying", "not-applied", "not-in-use", "orphaned", "unknown"], []),
        "configured_write_cache_policy": MoPropertyMeta("configured_write_cache_policy", "configuredWriteCachePolicy", "string", VersionMeta.Version221b, MoPropertyMeta.READ_ONLY, None, None, None, None, ["always-write-back", "unknown", "write-back-good-bbu", "write-through"], []),
        "connection_protocol": MoPropertyMeta("connection_protocol", "connectionProtocol", "string", VersionMeta.Version211a, MoPropertyMeta.READ_ONLY, None, None, None, None, ["NVME", "SAS", "SATA", "unspecified"], []),
        "deploy_action": MoPropertyMeta("deploy_action", "deployAction", "string", VersionMeta.Version224b, MoPropertyMeta.READ_ONLY, None, None, None, None, ["abort-replication", "create", "delete", "modify", "no-action", "replace", "restore", "set-offline", "set-online"], []),
        "descr": MoPropertyMeta("descr", "descr", "string", VersionMeta.Version251a, MoPropertyMeta.READ_ONLY, None, 0, 510, None, [], []),
        "dn": MoPropertyMeta("dn", "dn", "string", VersionMeta.Version211a, MoPropertyMeta.READ_ONLY, 0x20, 0, 256, None, [], []),
        "drive_cache": MoPropertyMeta("drive_cache", "driveCache", "string", VersionMeta.Version221b, MoPropertyMeta.READ_ONLY, None, None, None, None, ["disable", "enable", "no-change", "unknown"], []),
        "drive_security": MoPropertyMeta("drive_security", "driveSecurity", "string", VersionMeta.Version321d, MoPropertyMeta.READ_WRITE, 0x40, None, None, None, ["false", "no", "true", "yes"], []),
        "drive_state": MoPropertyMeta("drive_state", "driveState", "string", VersionMeta.Version221b, MoPropertyMeta.READ_ONLY, None, None, None, None, ["cache-degraded", "degraded", "offline", "optimal", "partially-degraded", "rebuilding", "unknown"], []),
        "id": MoPropertyMeta("id", "id", "string", VersionMeta.Version211a, MoPropertyMeta.NAMING, 0x80, None, None, None, ["unspecified"], ["0-4294967295"]),
        "io_policy": MoPropertyMeta("io_policy", "ioPolicy", "string", VersionMeta.Version221b, MoPropertyMeta.READ_ONLY, None, None, None, None, ["cached", "direct", "unknown"], []),
        "lc": MoPropertyMeta("lc", "lc", "string", VersionMeta.Version221b, MoPropertyMeta.READ_ONLY, None, None, None, None, ["allocated", "available", "deallocated", "repurposed"], []),
        "locale": MoPropertyMeta("locale", "locale", "string", VersionMeta.Version224b, MoPropertyMeta.READ_ONLY, None, None, None, r"""[\-\.:_a-zA-Z0-9]{0,16}""", [], []),
        "model": MoPropertyMeta("model", "model", "string", VersionMeta.Version211a, MoPropertyMeta.READ_ONLY, None, 0, 510, None, [], []),
        "name": MoPropertyMeta("name", "name", "string", VersionMeta.Version224b, MoPropertyMeta.READ_ONLY, None, 0, 510, None, [], []),
        "number_of_blocks": MoPropertyMeta("number_of_blocks", "numberOfBlocks", "string", VersionMeta.Version211a, MoPropertyMeta.READ_ONLY, None, None, None, None, ["unknown"], ["0-18446744073709551615"]),
        "oper_device_id": MoPropertyMeta("oper_device_id", "operDeviceId", "string", VersionMeta.Version224b, MoPropertyMeta.READ_ONLY, None, None, None, None, ["unspecified"], ["0-4294967295"]),
        "oper_qualifier_reason": MoPropertyMeta("oper_qualifier_reason", "operQualifierReason", "string", VersionMeta.Version211a, MoPropertyMeta.READ_ONLY, None, None, None, r"""[ !#$%&\(\)\*\+,\-\./:;\?@\[\]_\{\|\}~a-zA-Z0-9]{0,256}""", [], []),
        "oper_state": MoPropertyMeta("oper_state", "operState", "string", VersionMeta.Version224b, MoPropertyMeta.READ_ONLY, None, None, None, None, ["compute-degraded", "compute-inoperable", "offline", "online", "undefined"], []),
        "operability": MoPropertyMeta("operability", "operability", "string", VersionMeta.Version211a, MoPropertyMeta.READ_ONLY, None, None, None, None, ["accessibility-problem", "auto-upgrade", "backplane-port-problem", "bios-post-timeout", "chassis-intrusion", "chassis-limit-exceeded", "config", "decomissioning", "degraded", "disabled", "discovery", "discovery-failed", "equipment-problem", "fabric-conn-problem", "fabric-unsupported-conn", "identify", "identity-unestablishable", "inoperable", "link-activate-blocked", "malformed-fru", "non-optimal", "non-optimal-severe", "not-supported", "operable", "peer-comm-problem", "performance-problem", "post-failure", "power-problem", "powered-off", "removed", "thermal-problem", "unknown", "unsupported-config", "upgrade-problem", "voltage-problem"], []),
        "physical_block_size": MoPropertyMeta("physical_block_size", "physicalBlockSize", "string", VersionMeta.Version312b, MoPropertyMeta.READ_ONLY, None, None, None, None, ["512", "unknown"], ["0-4294967295"]),
        "pn_dn": MoPropertyMeta("pn_dn", "pnDn", "string", VersionMeta.Version251a, MoPropertyMeta.READ_ONLY, None, 0, 256, None, [], []),
        "presence": MoPropertyMeta("presence", "presence", "string", VersionMeta.Version211a, MoPropertyMeta.READ_ONLY, None, None, None, None, ["empty", "equipped", "equipped-deprecated", "equipped-disc-error", "equipped-disc-in-progress", "equipped-disc-not-started", "equipped-disc-unknown", "equipped-identity-unestablishable", "equipped-not-primary", "equipped-slave", "equipped-unsupported", "equipped-with-malformed-fru", "inaccessible", "mismatch", "mismatch-identity-unestablishable", "mismatch-slave", "missing", "missing-slave", "not-supported", "unauthorized", "unknown"], []),
        "read_policy": MoPropertyMeta("read_policy", "readPolicy", "string", VersionMeta.Version221b, MoPropertyMeta.READ_ONLY, None, None, None, None, ["normal", "read-ahead", "unknown"], []),
        "ref_dn": MoPropertyMeta("ref_dn", "refDn", "string", VersionMeta.Version251a, MoPropertyMeta.READ_ONLY, None, 0, 256, None, [], []),
        "revision": MoPropertyMeta("revision", "revision", "string", VersionMeta.Version211a, MoPropertyMeta.READ_ONLY, None, 0, 510, None, [], []),
        "rn": MoPropertyMeta("rn", "rn", "string", VersionMeta.Version211a, MoPropertyMeta.READ_ONLY, 0x100, 0, 256, None, [], []),
        "sacl": MoPropertyMeta("sacl", "sacl", "string", VersionMeta.Version302c, MoPropertyMeta.READ_ONLY, None, None, None, r"""((none|del|mod|addchild|cascade),){0,4}(none|del|mod|addchild|cascade){0,1}""", [], []),
        "security_flags": MoPropertyMeta("security_flags", "securityFlags", "string", VersionMeta.Version321d, MoPropertyMeta.READ_ONLY, None, None, None, r"""((defaultValue|none|driveSecurityCapable|driveSecurityEnable),){0,3}(defaultValue|none|driveSecurityCapable|driveSecurityEnable){0,1}""", [], []),
        "serial": MoPropertyMeta("serial", "serial", "string", VersionMeta.Version211a, MoPropertyMeta.READ_ONLY, None, 0, 510, None, [], []),
        "size": MoPropertyMeta("size", "size", "string", VersionMeta.Version211a, MoPropertyMeta.READ_ONLY, None, None, None, None, ["not-applicable"], ["0-18446744073709551615"]),
        "status": MoPropertyMeta("status", "status", "string", VersionMeta.Version211a, MoPropertyMeta.READ_WRITE, 0x200, None, None, r"""((removed|created|modified|deleted),){0,3}(removed|created|modified|deleted){0,1}""", [], []),
        "strip_size": MoPropertyMeta("strip_size", "stripSize", "string", VersionMeta.Version221b, MoPropertyMeta.READ_ONLY, None, None, None, None, ["unknown"], ["0-18446744073709551615"]),
        "type": MoPropertyMeta("type", "type", "string", VersionMeta.Version211a, MoPropertyMeta.READ_ONLY, None, None, None, None, ["mirror", "mirror-stripe", "raid", "simple", "stripe", "stripe-dual-parity", "stripe-dual-parity-stripe", "stripe-parity", "stripe-parity-stripe", "unspecified"], []),
        "uuid": MoPropertyMeta("uuid", "uuid", "string", VersionMeta.Version224b, MoPropertyMeta.READ_ONLY, None, None, None, r"""(([0-9a-fA-F]){8}\-([0-9a-fA-F]){4}\-([0-9a-fA-F]){4}\-([0-9a-fA-F]){4}\-([0-9a-fA-F]){12})|0""", [], []),
        "vendor": MoPropertyMeta("vendor", "vendor", "string", VersionMeta.Version211a, MoPropertyMeta.READ_ONLY, None, 0, 510, None, [], []),
        "vendor_uuid": MoPropertyMeta("vendor_uuid", "vendorUuid", "string", VersionMeta.Version224b, MoPropertyMeta.READ_ONLY, None, None, None, r"""(([0-9a-fA-F]){8}\-([0-9a-fA-F]){4}\-([0-9a-fA-F]){4}\-([0-9a-fA-F]){4}\-([0-9a-fA-F]){12})|0""", [], []),
    }

    prop_map = {
        "accessPolicy": "access_policy", 
        "actualWriteCachePolicy": "actual_write_cache_policy", 
        "adminActionTrigger": "admin_action_trigger", 
        "adminName": "admin_name", 
        "adminState": "admin_state", 
        "availableSize": "available_size", 
        "blockSize": "block_size", 
        "bootable": "bootable", 
        "changeId": "change_id", 
        "changeQualifier": "change_qualifier", 
        "childAction": "child_action", 
        "configQualifierReason": "config_qualifier_reason", 
        "configState": "config_state", 
        "configuredWriteCachePolicy": "configured_write_cache_policy", 
        "connectionProtocol": "connection_protocol", 
        "deployAction": "deploy_action", 
        "descr": "descr", 
        "dn": "dn", 
        "driveCache": "drive_cache", 
        "driveSecurity": "drive_security", 
        "driveState": "drive_state", 
        "id": "id", 
        "ioPolicy": "io_policy", 
        "lc": "lc", 
        "locale": "locale", 
        "model": "model", 
        "name": "name", 
        "numberOfBlocks": "number_of_blocks", 
        "operDeviceId": "oper_device_id", 
        "operQualifierReason": "oper_qualifier_reason", 
        "operState": "oper_state", 
        "operability": "operability", 
        "physicalBlockSize": "physical_block_size", 
        "pnDn": "pn_dn", 
        "presence": "presence", 
        "readPolicy": "read_policy", 
        "refDn": "ref_dn", 
        "revision": "revision", 
        "rn": "rn", 
        "sacl": "sacl", 
        "securityFlags": "security_flags", 
        "serial": "serial", 
        "size": "size", 
        "status": "status", 
        "stripSize": "strip_size", 
        "type": "type", 
        "uuid": "uuid", 
        "vendor": "vendor", 
        "vendorUuid": "vendor_uuid", 
    }

    def __init__(self, parent_mo_or_dn, id, **kwargs):
        self._dirty_mask = 0
        self.id = id
        self.access_policy = None
        self.actual_write_cache_policy = None
        self.admin_action_trigger = None
        self.admin_name = None
        self.admin_state = None
        self.available_size = None
        self.block_size = None
        self.bootable = None
        self.change_id = None
        self.change_qualifier = None
        self.child_action = None
        self.config_qualifier_reason = None
        self.config_state = None
        self.configured_write_cache_policy = None
        self.connection_protocol = None
        self.deploy_action = None
        self.descr = None
        self.drive_cache = None
        self.drive_security = None
        self.drive_state = None
        self.io_policy = None
        self.lc = None
        self.locale = None
        self.model = None
        self.name = None
        self.number_of_blocks = None
        self.oper_device_id = None
        self.oper_qualifier_reason = None
        self.oper_state = None
        self.operability = None
        self.physical_block_size = None
        self.pn_dn = None
        self.presence = None
        self.read_policy = None
        self.ref_dn = None
        self.revision = None
        self.sacl = None
        self.security_flags = None
        self.serial = None
        self.size = None
        self.status = None
        self.strip_size = None
        self.type = None
        self.uuid = None
        self.vendor = None
        self.vendor_uuid = None

        ManagedObject.__init__(self, "StorageVirtualDrive", parent_mo_or_dn, **kwargs)



"""This module contains the general information for ApeSANBoot ManagedObject."""

from ...ucsmo import ManagedObject
from ...ucscoremeta import MoPropertyMeta, MoMeta
from ...ucsmeta import VersionMeta


class ApeSANBootConsts:
    IS_HOST_AGENT_PRESENT_FALSE = "false"
    IS_HOST_AGENT_PRESENT_NO = "no"
    IS_HOST_AGENT_PRESENT_TRUE = "true"
    IS_HOST_AGENT_PRESENT_YES = "yes"
    TARGET_LUN_UNSPECIFIED = "unspecified"
    TYPE_LINUX = "Linux"
    TYPE_PNU_OS = "PnuOS"
    TYPE_SOLARIS = "Solaris"
    TYPE_VMWARE_ESX = "VMWareESX"
    TYPE_WINDOWS = "Windows"
    TYPE_UNSPECIFIED = "unspecified"


class ApeSANBoot(ManagedObject):
    """This is ApeSANBoot class."""

    consts = ApeSANBootConsts()
    naming_props = set(['targetWWN'])

    mo_meta = MoMeta("ApeSANBoot", "apeSANBoot", "sanboot-[target_wwn]", VersionMeta.Version101e, "InputOutput", 0x7f, [], ["admin"], ['apeManager'], [], [None])

    prop_meta = {
        "child_action": MoPropertyMeta("child_action", "childAction", "string", VersionMeta.Version101e, MoPropertyMeta.INTERNAL, 0x2, None, None, r"""((deleteAll|ignore|deleteNonPresent),){0,2}(deleteAll|ignore|deleteNonPresent){0,1}""", [], []),
        "dn": MoPropertyMeta("dn", "dn", "string", VersionMeta.Version101e, MoPropertyMeta.READ_ONLY, 0x4, 0, 256, None, [], []),
        "hostname": MoPropertyMeta("hostname", "hostname", "string", VersionMeta.Version101e, MoPropertyMeta.READ_ONLY, None, None, None, None, [], []),
        "is_host_agent_present": MoPropertyMeta("is_host_agent_present", "isHostAgentPresent", "string", VersionMeta.Version101e, MoPropertyMeta.READ_ONLY, None, None, None, None, ["false", "no", "true", "yes"], []),
        "kernel_name": MoPropertyMeta("kernel_name", "kernelName", "string", VersionMeta.Version101e, MoPropertyMeta.READ_ONLY, None, None, None, None, [], []),
        "kernel_release": MoPropertyMeta("kernel_release", "kernelRelease", "string", VersionMeta.Version101e, MoPropertyMeta.READ_ONLY, None, None, None, None, [], []),
        "kernel_version": MoPropertyMeta("kernel_version", "kernelVersion", "string", VersionMeta.Version101e, MoPropertyMeta.READ_ONLY, None, None, None, None, [], []),
        "name": MoPropertyMeta("name", "name", "string", VersionMeta.Version101e, MoPropertyMeta.READ_ONLY, None, None, None, None, [], []),
        "rn": MoPropertyMeta("rn", "rn", "string", VersionMeta.Version101e, MoPropertyMeta.READ_ONLY, 0x8, 0, 256, None, [], []),
        "sacl": MoPropertyMeta("sacl", "sacl", "string", VersionMeta.Version302c, MoPropertyMeta.READ_ONLY, None, None, None, r"""((none|del|mod|addchild|cascade),){0,4}(none|del|mod|addchild|cascade){0,1}""", [], []),
        "status": MoPropertyMeta("status", "status", "string", VersionMeta.Version101e, MoPropertyMeta.READ_WRITE, 0x10, None, None, r"""((removed|created|modified|deleted),){0,3}(removed|created|modified|deleted){0,1}""", [], []),
        "target_lun": MoPropertyMeta("target_lun", "targetLun", "string", VersionMeta.Version101e, MoPropertyMeta.READ_WRITE, 0x20, None, None, None, ["unspecified"], ["0-4294967295"]),
        "target_wwn": MoPropertyMeta("target_wwn", "targetWWN", "string", VersionMeta.Version101e, MoPropertyMeta.NAMING, 0x40, 0, 256, r"""(([A-Fa-f0-9][A-Fa-f0-9]:){7}[A-Fa-f0-9][A-Fa-f0-9])|0""", [], []),
        "type": MoPropertyMeta("type", "type", "string", VersionMeta.Version101e, MoPropertyMeta.READ_ONLY, None, None, None, None, ["Linux", "PnuOS", "Solaris", "VMWareESX", "Windows", "unspecified"], []),
    }

    prop_map = {
        "childAction": "child_action", 
        "dn": "dn", 
        "hostname": "hostname", 
        "isHostAgentPresent": "is_host_agent_present", 
        "kernelName": "kernel_name", 
        "kernelRelease": "kernel_release", 
        "kernelVersion": "kernel_version", 
        "name": "name", 
        "rn": "rn", 
        "sacl": "sacl", 
        "status": "status", 
        "targetLun": "target_lun", 
        "targetWWN": "target_wwn", 
        "type": "type", 
    }

    def __init__(self, parent_mo_or_dn, target_wwn, **kwargs):
        self._dirty_mask = 0
        self.target_wwn = target_wwn
        self.child_action = None
        self.hostname = None
        self.is_host_agent_present = None
        self.kernel_name = None
        self.kernel_release = None
        self.kernel_version = None
        self.name = None
        self.sacl = None
        self.status = None
        self.target_lun = None
        self.type = None

        ManagedObject.__init__(self, "ApeSANBoot", parent_mo_or_dn, **kwargs)



"""This module contains the general information for MemoryPersistentMemoryNamespaceConfigResult ManagedObject."""

from ...ucsmo import ManagedObject
from ...ucscoremeta import MoPropertyMeta, MoMeta
from ...ucsmeta import VersionMeta


class MemoryPersistentMemoryNamespaceConfigResultConsts:
    SOCKET_ID_SOCKET_1 = "socket-1"
    SOCKET_ID_SOCKET_2 = "socket-2"
    SOCKET_ID_SOCKET_3 = "socket-3"
    SOCKET_ID_SOCKET_4 = "socket-4"
    SOCKET_LOCAL_DIMM_NUMBER_10 = "10"
    SOCKET_LOCAL_DIMM_NUMBER_11 = "11"
    SOCKET_LOCAL_DIMM_NUMBER_12 = "12"
    SOCKET_LOCAL_DIMM_NUMBER_14 = "14"
    SOCKET_LOCAL_DIMM_NUMBER_15 = "15"
    SOCKET_LOCAL_DIMM_NUMBER_16 = "16"
    SOCKET_LOCAL_DIMM_NUMBER_2 = "2"
    SOCKET_LOCAL_DIMM_NUMBER_3 = "3"
    SOCKET_LOCAL_DIMM_NUMBER_4 = "4"
    SOCKET_LOCAL_DIMM_NUMBER_6 = "6"
    SOCKET_LOCAL_DIMM_NUMBER_7 = "7"
    SOCKET_LOCAL_DIMM_NUMBER_8 = "8"
    SOCKET_LOCAL_DIMM_NUMBER_NOT_APPLICABLE = "not-applicable"


class MemoryPersistentMemoryNamespaceConfigResult(ManagedObject):
    """This is MemoryPersistentMemoryNamespaceConfigResult class."""

    consts = MemoryPersistentMemoryNamespaceConfigResultConsts()
    naming_props = set(['name'])

    mo_meta = MoMeta("MemoryPersistentMemoryNamespaceConfigResult", "memoryPersistentMemoryNamespaceConfigResult", "nscr-[name]", VersionMeta.Version404a, "InputOutput", 0x3f, [], ["admin", "pn-equipment", "pn-maintenance", "pn-policy"], ['memoryPersistentMemoryConfigResult'], ['faultInst'], [None])

    prop_meta = {
        "child_action": MoPropertyMeta("child_action", "childAction", "string", VersionMeta.Version404a, MoPropertyMeta.INTERNAL, 0x2, None, None, r"""((deleteAll|ignore|deleteNonPresent),){0,2}(deleteAll|ignore|deleteNonPresent){0,1}""", [], []),
        "config_status": MoPropertyMeta("config_status", "configStatus", "string", VersionMeta.Version404a, MoPropertyMeta.READ_ONLY, None, 0, 510, None, [], []),
        "dn": MoPropertyMeta("dn", "dn", "string", VersionMeta.Version404a, MoPropertyMeta.READ_ONLY, 0x4, 0, 256, None, [], []),
        "name": MoPropertyMeta("name", "name", "string", VersionMeta.Version404a, MoPropertyMeta.NAMING, 0x8, 1, 510, None, [], []),
        "rn": MoPropertyMeta("rn", "rn", "string", VersionMeta.Version404a, MoPropertyMeta.READ_ONLY, 0x10, 0, 256, None, [], []),
        "sacl": MoPropertyMeta("sacl", "sacl", "string", VersionMeta.Version404a, MoPropertyMeta.READ_ONLY, None, None, None, r"""((none|del|mod|addchild|cascade),){0,4}(none|del|mod|addchild|cascade){0,1}""", [], []),
        "socket_id": MoPropertyMeta("socket_id", "socketId", "string", VersionMeta.Version404a, MoPropertyMeta.READ_ONLY, None, None, None, None, ["socket-1", "socket-2", "socket-3", "socket-4"], []),
        "socket_local_dimm_number": MoPropertyMeta("socket_local_dimm_number", "socketLocalDimmNumber", "string", VersionMeta.Version404a, MoPropertyMeta.READ_ONLY, None, None, None, None, ["10", "11", "12", "14", "15", "16", "2", "3", "4", "6", "7", "8", "not-applicable"], []),
        "status": MoPropertyMeta("status", "status", "string", VersionMeta.Version404a, MoPropertyMeta.READ_WRITE, 0x20, None, None, r"""((removed|created|modified|deleted),){0,3}(removed|created|modified|deleted){0,1}""", [], []),
    }

    prop_map = {
        "childAction": "child_action", 
        "configStatus": "config_status", 
        "dn": "dn", 
        "name": "name", 
        "rn": "rn", 
        "sacl": "sacl", 
        "socketId": "socket_id", 
        "socketLocalDimmNumber": "socket_local_dimm_number", 
        "status": "status", 
    }

    def __init__(self, parent_mo_or_dn, name, **kwargs):
        self._dirty_mask = 0
        self.name = name
        self.child_action = None
        self.config_status = None
        self.sacl = None
        self.socket_id = None
        self.socket_local_dimm_number = None
        self.status = None

        ManagedObject.__init__(self, "MemoryPersistentMemoryNamespaceConfigResult", parent_mo_or_dn, **kwargs)



"""This module contains the general information for SysdebugLogControlDomain ManagedObject."""

from ...ucsmo import ManagedObject
from ...ucscoremeta import MoPropertyMeta, MoMeta
from ...ucsmeta import VersionMeta


class SysdebugLogControlDomainConsts:
    LEVEL_CRIT = "crit"
    LEVEL_DEBUG0 = "debug0"
    LEVEL_DEBUG1 = "debug1"
    LEVEL_DEBUG2 = "debug2"
    LEVEL_DEBUG3 = "debug3"
    LEVEL_DEBUG4 = "debug4"
    LEVEL_INFO = "info"
    LEVEL_MAJOR = "major"
    LEVEL_MINOR = "minor"
    LEVEL_WARN = "warn"
    LEVEL_FLAG_FALSE = "false"
    LEVEL_FLAG_NO = "no"
    LEVEL_FLAG_TRUE = "true"
    LEVEL_FLAG_YES = "yes"
    NAME_SYSMGMT = "sysmgmt"
    PERSIST_FALSE = "false"
    PERSIST_NO = "no"
    PERSIST_TRUE = "true"
    PERSIST_YES = "yes"
    RESET_FALSE = "false"
    RESET_NO = "no"
    RESET_TRUE = "true"
    RESET_YES = "yes"


class SysdebugLogControlDomain(ManagedObject):
    """This is SysdebugLogControlDomain class."""

    consts = SysdebugLogControlDomainConsts()
    naming_props = set(['name'])

    mo_meta = MoMeta("SysdebugLogControlDomain", "sysdebugLogControlDomain", "domain-[name]", VersionMeta.Version101e, "InputOutput", 0x3ff, [], ["admin", "operations"], ['sysdebugLogControlEp'], ['sysdebugLogControlDestinationFile', 'sysdebugLogControlDestinationSyslog', 'sysdebugLogControlModule'], ["Get"])

    prop_meta = {
        "child_action": MoPropertyMeta("child_action", "childAction", "string", VersionMeta.Version101e, MoPropertyMeta.INTERNAL, 0x2, None, None, r"""((deleteAll|ignore|deleteNonPresent),){0,2}(deleteAll|ignore|deleteNonPresent){0,1}""", [], []),
        "dn": MoPropertyMeta("dn", "dn", "string", VersionMeta.Version101e, MoPropertyMeta.READ_ONLY, 0x4, 0, 256, None, [], []),
        "level": MoPropertyMeta("level", "level", "string", VersionMeta.Version101e, MoPropertyMeta.READ_WRITE, 0x8, None, None, None, ["crit", "debug0", "debug1", "debug2", "debug3", "debug4", "info", "major", "minor", "warn"], []),
        "level_flag": MoPropertyMeta("level_flag", "levelFlag", "string", VersionMeta.Version101e, MoPropertyMeta.READ_WRITE, 0x10, None, None, None, ["false", "no", "true", "yes"], []),
        "name": MoPropertyMeta("name", "name", "string", VersionMeta.Version101e, MoPropertyMeta.NAMING, 0x20, None, None, None, ["sysmgmt"], []),
        "persist": MoPropertyMeta("persist", "persist", "string", VersionMeta.Version101e, MoPropertyMeta.READ_WRITE, 0x40, None, None, None, ["false", "no", "true", "yes"], []),
        "reset": MoPropertyMeta("reset", "reset", "string", VersionMeta.Version101e, MoPropertyMeta.READ_WRITE, 0x80, None, None, None, ["false", "no", "true", "yes"], []),
        "rn": MoPropertyMeta("rn", "rn", "string", VersionMeta.Version101e, MoPropertyMeta.READ_ONLY, 0x100, 0, 256, None, [], []),
        "sacl": MoPropertyMeta("sacl", "sacl", "string", VersionMeta.Version302c, MoPropertyMeta.READ_ONLY, None, None, None, r"""((none|del|mod|addchild|cascade),){0,4}(none|del|mod|addchild|cascade){0,1}""", [], []),
        "status": MoPropertyMeta("status", "status", "string", VersionMeta.Version101e, MoPropertyMeta.READ_WRITE, 0x200, None, None, r"""((removed|created|modified|deleted),){0,3}(removed|created|modified|deleted){0,1}""", [], []),
    }

    prop_map = {
        "childAction": "child_action", 
        "dn": "dn", 
        "level": "level", 
        "levelFlag": "level_flag", 
        "name": "name", 
        "persist": "persist", 
        "reset": "reset", 
        "rn": "rn", 
        "sacl": "sacl", 
        "status": "status", 
    }

    def __init__(self, parent_mo_or_dn, name, **kwargs):
        self._dirty_mask = 0
        self.name = name
        self.child_action = None
        self.level = None
        self.level_flag = None
        self.persist = None
        self.reset = None
        self.sacl = None
        self.status = None

        ManagedObject.__init__(self, "SysdebugLogControlDomain", parent_mo_or_dn, **kwargs)



"""This module contains the general information for BiosSettings ManagedObject."""

from ...ucsmo import ManagedObject
from ...ucscoremeta import MoPropertyMeta, MoMeta
from ...ucsmeta import VersionMeta


class BiosSettingsConsts:
    pass


class BiosSettings(ManagedObject):
    """This is BiosSettings class."""

    consts = BiosSettingsConsts()
    naming_props = set([])

    mo_meta = MoMeta("BiosSettings", "biosSettings", "bios-settings", VersionMeta.Version131c, "InputOutput", 0x1f, [], [""], ['biosUnit', 'capabilityCatalogue', 'computeDefaults', 'computePlatform'], ['biosTokenFeatureGroup', 'biosTokenParam', 'biosVfACPI10Support', 'biosVfASPMSupport', 'biosVfAllUSBDevices', 'biosVfAltitude', 'biosVfAssertNMIOnPERR', 'biosVfAssertNMIOnSERR', 'biosVfBMEDMAMitigation', 'biosVfBootOptionRetry', 'biosVfCPUHardwarePowerManagement', 'biosVfCPUPerformance', 'biosVfConsistentDeviceNameControl', 'biosVfConsoleRedirection', 'biosVfCoreMultiProcessing', 'biosVfDDR3VoltageSelection', 'biosVfDRAMClockThrottling', 'biosVfDirectCacheAccess', 'biosVfDramRefreshRate', 'biosVfEnergyPerformanceTuning', 'biosVfEnhancedIntelSpeedStepTech', 'biosVfEnhancedPowerCappingSupport', 'biosVfExecuteDisableBit', 'biosVfFRB2Timer', 'biosVfFrequencyFloorOverride', 'biosVfFrontPanelLockout', 'biosVfIOEMezz1OptionROM', 'biosVfIOENVMe1OptionROM', 'biosVfIOENVMe2OptionROM', 'biosVfIOESlot1OptionROM', 'biosVfIOESlot2OptionROM', 'biosVfIntegratedGraphics', 'biosVfIntegratedGraphicsApertureSize', 'biosVfIntelEntrySASRAIDModule', 'biosVfIntelHyperThreadingTech', 'biosVfIntelTrustedExecutionTechnology', 'biosVfIntelTurboBoostTech', 'biosVfIntelVTForDirectedIO', 'biosVfIntelVirtualizationTechnology', 'biosVfInterleaveConfiguration', 'biosVfLocalX2Apic', 'biosVfLvDIMMSupport', 'biosVfMaxVariableMTRRSetting', 'biosVfMaximumMemoryBelow4GB', 'biosVfMemoryMappedIOAbove4GB', 'biosVfMirroringMode', 'biosVfNUMAOptimized', 'biosVfOSBootWatchdogTimer', 'biosVfOSBootWatchdogTimerPolicy', 'biosVfOSBootWatchdogTimerTimeout', 'biosVfOnboardGraphics', 'biosVfOnboardSATAController', 'biosVfOnboardStorage', 'biosVfOptionROMEnable', 'biosVfOptionROMLoad', 'biosVfOutOfBandManagement', 'biosVfPCHSATAMode', 'biosVfPCILOMPortsConfiguration', 'biosVfPCIROMCLP', 'biosVfPCISlotLinkSpeed', 'biosVfPCISlotOptionROMEnable', 'biosVfPOSTErrorPause', 'biosVfPSTATECoordination', 'biosVfPackageCStateLimit', 'biosVfPanicAndHighWatermark', 'biosVfProcessorC1E', 'biosVfProcessorC3Report', 'biosVfProcessorC6Report', 'biosVfProcessorC7Report', 'biosVfProcessorCMCI', 'biosVfProcessorCState', 'biosVfProcessorEnergyConfiguration', 'biosVfProcessorPrefetchConfig', 'biosVfQPILinkFrequencySelect', 'biosVfQPISnoopMode', 'biosVfQuietBoot', 'biosVfRedirectionAfterBIOSPOST', 'biosVfResumeOnACPowerLoss', 'biosVfSBMezz1OptionROM', 'biosVfSBNVMe1OptionROM', 'biosVfSIOC1OptionROM', 'biosVfSIOC2OptionROM', 'biosVfScrubPolicies', 'biosVfSelectMemoryRASConfiguration', 'biosVfSerialPortAEnable', 'biosVfSparingMode', 'biosVfSriovConfig', 'biosVfTPMPendingOperation', 'biosVfTPMSupport', 'biosVfTrustedPlatformModule', 'biosVfUCSMBootModeControl', 'biosVfUCSMBootOrderRuleControl', 'biosVfUEFIOSUseLegacyVideo', 'biosVfUSBBootConfig', 'biosVfUSBConfiguration', 'biosVfUSBFrontPanelAccessLock', 'biosVfUSBPortConfiguration', 'biosVfUSBSystemIdlePowerOptimizingSetting', 'biosVfVGAPriority', 'biosVfWorkloadConfiguration'], ["Get"])

    prop_meta = {
        "child_action": MoPropertyMeta("child_action", "childAction", "string", VersionMeta.Version131c, MoPropertyMeta.INTERNAL, 0x2, None, None, r"""((deleteAll|ignore|deleteNonPresent),){0,2}(deleteAll|ignore|deleteNonPresent){0,1}""", [], []),
        "dn": MoPropertyMeta("dn", "dn", "string", VersionMeta.Version131c, MoPropertyMeta.READ_ONLY, 0x4, 0, 256, None, [], []),
        "prop_acl": MoPropertyMeta("prop_acl", "propAcl", "ulong", VersionMeta.Version302c, MoPropertyMeta.READ_ONLY, None, None, None, None, [], []),
        "rn": MoPropertyMeta("rn", "rn", "string", VersionMeta.Version131c, MoPropertyMeta.READ_ONLY, 0x8, 0, 256, None, [], []),
        "sacl": MoPropertyMeta("sacl", "sacl", "string", VersionMeta.Version302c, MoPropertyMeta.READ_ONLY, None, None, None, r"""((none|del|mod|addchild|cascade),){0,4}(none|del|mod|addchild|cascade){0,1}""", [], []),
        "status": MoPropertyMeta("status", "status", "string", VersionMeta.Version131c, MoPropertyMeta.READ_WRITE, 0x10, None, None, r"""((removed|created|modified|deleted),){0,3}(removed|created|modified|deleted){0,1}""", [], []),
    }

    prop_map = {
        "childAction": "child_action", 
        "dn": "dn", 
        "propAcl": "prop_acl", 
        "rn": "rn", 
        "sacl": "sacl", 
        "status": "status", 
    }

    def __init__(self, parent_mo_or_dn, **kwargs):
        self._dirty_mask = 0
        self.child_action = None
        self.prop_acl = None
        self.sacl = None
        self.status = None

        ManagedObject.__init__(self, "BiosSettings", parent_mo_or_dn, **kwargs)



"""This module contains the general information for ComputePhysicalFsmTask ManagedObject."""

from ...ucsmo import ManagedObject
from ...ucscoremeta import MoPropertyMeta, MoMeta
from ...ucsmeta import VersionMeta


class ComputePhysicalFsmTaskConsts:
    COMPLETION_CANCELLED = "cancelled"
    COMPLETION_COMPLETED = "completed"
    COMPLETION_PROCESSING = "processing"
    COMPLETION_SCHEDULED = "scheduled"
    ITEM_ACTIVATE_ADAPTOR = "ActivateAdaptor"
    ITEM_ACTIVATE_BIOS = "ActivateBIOS"
    ITEM_ASSOCIATE = "Associate"
    ITEM_BIOS_RECOVERY = "BiosRecovery"
    ITEM_CIMC_SECURITY_CONFIG = "CimcSecurityConfig"
    ITEM_CIMC_SESSION_DELETE = "CimcSessionDelete"
    ITEM_CMOS_RESET = "CmosReset"
    ITEM_CONFIG_BOARD = "ConfigBoard"
    ITEM_CONFIG_SO_L = "ConfigSoL"
    ITEM_CONFIGURE_SERVER_PERSONALITY = "ConfigureServerPersonality"
    ITEM_DECOMMISSION = "Decommission"
    ITEM_DIAG = "Diag"
    ITEM_DIAGNOSTIC_INTERRUPT = "DiagnosticInterrupt"
    ITEM_DISASSOCIATE = "Disassociate"
    ITEM_DISK_ZONING_INVENTORY = "DiskZoningInventory"
    ITEM_ENABLE_CIMC_SECURE_BOOT = "EnableCimcSecureBoot"
    ITEM_EXECUTE_ACTIONS = "ExecuteActions"
    ITEM_FLASH_CONTROLLER = "FlashController"
    ITEM_FW_UPGRADE = "FwUpgrade"
    ITEM_HARD_SHUTDOWN = "HardShutdown"
    ITEM_HARDRESET = "Hardreset"
    ITEM_NVME_SWITCH_RECOVERY = "NvmeSwitchRecovery"
    ITEM_OOB_STORAGE_ADMIN_CONFIG = "OobStorageAdminConfig"
    ITEM_POWER_CAP = "PowerCap"
    ITEM_POWERCYCLE = "Powercycle"
    ITEM_REINITIALIZE_VIRTUAL_DRIVE = "ReinitializeVirtualDrive"
    ITEM_RESET_BMC = "ResetBmc"
    ITEM_RESET_IPMI = "ResetIpmi"
    ITEM_RESET_KVM = "ResetKvm"
    ITEM_RESET_MEMORY_ERRORS = "ResetMemoryErrors"
    ITEM_SERVICE_INFRA_DEPLOY = "ServiceInfraDeploy"
    ITEM_SERVICE_INFRA_WITHDRAW = "ServiceInfraWithdraw"
    ITEM_SOFT_SHUTDOWN = "SoftShutdown"
    ITEM_SOFTRESET = "Softreset"
    ITEM_SW_CONN_UPD = "SwConnUpd"
    ITEM_TURNUP = "Turnup"
    ITEM_UNCONFIG_SO_L = "UnconfigSoL"
    ITEM_UPDATE_ADAPTOR = "UpdateAdaptor"
    ITEM_UPDATE_BIOS = "UpdateBIOS"
    ITEM_UPDATE_BOARD_CONTROLLER = "UpdateBoardController"
    ITEM_CLEAR_TPM = "clearTPM"
    ITEM_NOP = "nop"
    ITEM_UPDATE_EXT_USERS = "updateExtUsers"


class ComputePhysicalFsmTask(ManagedObject):
    """This is ComputePhysicalFsmTask class."""

    consts = ComputePhysicalFsmTaskConsts()
    naming_props = set(['item'])

    mo_meta = MoMeta("ComputePhysicalFsmTask", "computePhysicalFsmTask", "task-[item]", VersionMeta.Version141i, "OutputOnly", 0xf, [], [""], ['computeBlade', 'computeRackUnit', 'computeServerUnit'], [], [None])

    prop_meta = {
        "child_action": MoPropertyMeta("child_action", "childAction", "string", VersionMeta.Version141i, MoPropertyMeta.INTERNAL, None, None, None, r"""((deleteAll|ignore|deleteNonPresent),){0,2}(deleteAll|ignore|deleteNonPresent){0,1}""", [], []),
        "completion": MoPropertyMeta("completion", "completion", "string", VersionMeta.Version141i, MoPropertyMeta.READ_ONLY, None, None, None, None, ["cancelled", "completed", "processing", "scheduled"], []),
        "dn": MoPropertyMeta("dn", "dn", "string", VersionMeta.Version141i, MoPropertyMeta.READ_ONLY, 0x2, 0, 256, None, [], []),
        "flags": MoPropertyMeta("flags", "flags", "string", VersionMeta.Version141i, MoPropertyMeta.READ_ONLY, None, None, None, r"""(defaultValue){0,1}""", [], []),
        "item": MoPropertyMeta("item", "item", "string", VersionMeta.Version141i, MoPropertyMeta.NAMING, None, None, None, None, ["ActivateAdaptor", "ActivateBIOS", "Associate", "BiosRecovery", "CimcSecurityConfig", "CimcSessionDelete", "CmosReset", "ConfigBoard", "ConfigSoL", "ConfigureServerPersonality", "Decommission", "Diag", "DiagnosticInterrupt", "Disassociate", "DiskZoningInventory", "EnableCimcSecureBoot", "ExecuteActions", "FlashController", "FwUpgrade", "HardShutdown", "Hardreset", "NvmeSwitchRecovery", "OobStorageAdminConfig", "PowerCap", "Powercycle", "ReinitializeVirtualDrive", "ResetBmc", "ResetIpmi", "ResetKvm", "ResetMemoryErrors", "ServiceInfraDeploy", "ServiceInfraWithdraw", "SoftShutdown", "Softreset", "SwConnUpd", "Turnup", "UnconfigSoL", "UpdateAdaptor", "UpdateBIOS", "UpdateBoardController", "clearTPM", "nop", "updateExtUsers"], []),
        "rn": MoPropertyMeta("rn", "rn", "string", VersionMeta.Version141i, MoPropertyMeta.READ_ONLY, 0x4, 0, 256, None, [], []),
        "sacl": MoPropertyMeta("sacl", "sacl", "string", VersionMeta.Version302c, MoPropertyMeta.READ_ONLY, None, None, None, r"""((none|del|mod|addchild|cascade),){0,4}(none|del|mod|addchild|cascade){0,1}""", [], []),
        "seq_id": MoPropertyMeta("seq_id", "seqId", "uint", VersionMeta.Version141i, MoPropertyMeta.READ_ONLY, None, None, None, None, [], []),
        "status": MoPropertyMeta("status", "status", "string", VersionMeta.Version141i, MoPropertyMeta.READ_WRITE, 0x8, None, None, r"""((removed|created|modified|deleted),){0,3}(removed|created|modified|deleted){0,1}""", [], []),
    }

    prop_map = {
        "childAction": "child_action", 
        "completion": "completion", 
        "dn": "dn", 
        "flags": "flags", 
        "item": "item", 
        "rn": "rn", 
        "sacl": "sacl", 
        "seqId": "seq_id", 
        "status": "status", 
    }

    def __init__(self, parent_mo_or_dn, item, **kwargs):
        self._dirty_mask = 0
        self.item = item
        self.child_action = None
        self.completion = None
        self.flags = None
        self.sacl = None
        self.seq_id = None
        self.status = None

        ManagedObject.__init__(self, "ComputePhysicalFsmTask", parent_mo_or_dn, **kwargs)



"""This module contains the general information for MoIpV4AddrKv ManagedObject."""

from ...ucsmo import ManagedObject
from ...ucscoremeta import MoPropertyMeta, MoMeta
from ...ucsmeta import VersionMeta


class MoIpV4AddrKvConsts:
    OWNER_INVENTORY = "inventory"
    OWNER_MGMT = "mgmt"
    TYPE_IPV4_ADDR = "ipv4-addr"
    TYPE_IPV6_ADDR = "ipv6-addr"
    TYPE_KEY_VALUE = "key-value"
    TYPE_VLAN = "vlan"
    TYPE_VNIC = "vnic"


class MoIpV4AddrKv(ManagedObject):
    """This is MoIpV4AddrKv class."""

    consts = MoIpV4AddrKvConsts()
    naming_props = set(['key'])

    mo_meta = MoMeta("MoIpV4AddrKv", "moIpV4AddrKv", "kv-[key]", VersionMeta.Version321d, "InputOutput", 0x3ff, [], ["admin", "read-only"], ['moKvCfgHolder'], ['faultInst'], ["Get", "Set"])

    prop_meta = {
        "addr": MoPropertyMeta("addr", "addr", "string", VersionMeta.Version321d, MoPropertyMeta.READ_ONLY, None, 0, 256, r"""((([0-9]){1,3}\.){3}[0-9]{1,3})""", [], []),
        "child_action": MoPropertyMeta("child_action", "childAction", "string", VersionMeta.Version321d, MoPropertyMeta.INTERNAL, 0x2, None, None, r"""((deleteAll|ignore|deleteNonPresent),){0,2}(deleteAll|ignore|deleteNonPresent){0,1}""", [], []),
        "def_gw": MoPropertyMeta("def_gw", "defGw", "string", VersionMeta.Version321d, MoPropertyMeta.READ_ONLY, None, 0, 256, r"""((([0-9]){1,3}\.){3}[0-9]{1,3})""", [], []),
        "dn": MoPropertyMeta("dn", "dn", "string", VersionMeta.Version321d, MoPropertyMeta.READ_ONLY, 0x4, 0, 256, None, [], []),
        "key": MoPropertyMeta("key", "key", "string", VersionMeta.Version321d, MoPropertyMeta.NAMING, 0x8, 1, 510, None, [], []),
        "oper_pool_name": MoPropertyMeta("oper_pool_name", "operPoolName", "string", VersionMeta.Version321d, MoPropertyMeta.READ_ONLY, None, 0, 256, None, [], []),
        "owner": MoPropertyMeta("owner", "owner", "string", VersionMeta.Version321d, MoPropertyMeta.READ_ONLY, None, None, None, None, ["inventory", "mgmt"], []),
        "pool_name": MoPropertyMeta("pool_name", "poolName", "string", VersionMeta.Version321d, MoPropertyMeta.READ_WRITE, 0x10, None, None, None, [], []),
        "prim_dns": MoPropertyMeta("prim_dns", "primDns", "string", VersionMeta.Version321d, MoPropertyMeta.READ_WRITE, 0x20, 0, 256, r"""((([0-9]){1,3}\.){3}[0-9]{1,3})""", [], []),
        "rn": MoPropertyMeta("rn", "rn", "string", VersionMeta.Version321d, MoPropertyMeta.READ_ONLY, 0x40, 0, 256, None, [], []),
        "sacl": MoPropertyMeta("sacl", "sacl", "string", VersionMeta.Version321d, MoPropertyMeta.READ_ONLY, None, None, None, r"""((none|del|mod|addchild|cascade),){0,4}(none|del|mod|addchild|cascade){0,1}""", [], []),
        "sec_dns": MoPropertyMeta("sec_dns", "secDns", "string", VersionMeta.Version321d, MoPropertyMeta.READ_WRITE, 0x80, 0, 256, r"""((([0-9]){1,3}\.){3}[0-9]{1,3})""", [], []),
        "status": MoPropertyMeta("status", "status", "string", VersionMeta.Version321d, MoPropertyMeta.READ_WRITE, 0x100, None, None, r"""((removed|created|modified|deleted),){0,3}(removed|created|modified|deleted){0,1}""", [], []),
        "subnet": MoPropertyMeta("subnet", "subnet", "string", VersionMeta.Version321d, MoPropertyMeta.READ_ONLY, None, 0, 256, r"""((([0-9]){1,3}\.){3}[0-9]{1,3})""", [], []),
        "target": MoPropertyMeta("target", "target", "string", VersionMeta.Version321d, MoPropertyMeta.READ_WRITE, 0x200, None, None, r"""((defaultValue|host|mgmt),){0,2}(defaultValue|host|mgmt){0,1}""", [], []),
        "type": MoPropertyMeta("type", "type", "string", VersionMeta.Version321d, MoPropertyMeta.READ_ONLY, None, None, None, None, ["ipv4-addr", "ipv6-addr", "key-value", "vlan", "vnic"], []),
        "value": MoPropertyMeta("value", "value", "string", VersionMeta.Version321d, MoPropertyMeta.READ_ONLY, None, 0, 510, None, [], []),
    }

    prop_map = {
        "addr": "addr", 
        "childAction": "child_action", 
        "defGw": "def_gw", 
        "dn": "dn", 
        "key": "key", 
        "operPoolName": "oper_pool_name", 
        "owner": "owner", 
        "poolName": "pool_name", 
        "primDns": "prim_dns", 
        "rn": "rn", 
        "sacl": "sacl", 
        "secDns": "sec_dns", 
        "status": "status", 
        "subnet": "subnet", 
        "target": "target", 
        "type": "type", 
        "value": "value", 
    }

    def __init__(self, parent_mo_or_dn, key, **kwargs):
        self._dirty_mask = 0
        self.key = key
        self.addr = None
        self.child_action = None
        self.def_gw = None
        self.oper_pool_name = None
        self.owner = None
        self.pool_name = None
        self.prim_dns = None
        self.sacl = None
        self.sec_dns = None
        self.status = None
        self.subnet = None
        self.target = None
        self.type = None
        self.value = None

        ManagedObject.__init__(self, "MoIpV4AddrKv", parent_mo_or_dn, **kwargs)



"""This module contains the general information for FirmwareUpdatable ManagedObject."""

from ...ucsmo import ManagedObject
from ...ucscoremeta import MoPropertyMeta, MoMeta
from ...ucsmeta import VersionMeta


class FirmwareUpdatableConsts:
    ADMIN_STATE_FORCE_TRIGGER = "force-trigger"
    ADMIN_STATE_TRIGGER = "trigger"
    ADMIN_STATE_TRIGGERED = "triggered"
    DEPLOYMENT_BACKUP = "backup"
    DEPLOYMENT_UNSPECIFIED = "unspecified"
    OPER_STATE_ACTIVATING = "activating"
    OPER_STATE_AUTO_ACTIVATING = "auto-activating"
    OPER_STATE_AUTO_UPDATING = "auto-updating"
    OPER_STATE_BAD_IMAGE = "bad-image"
    OPER_STATE_FAILED = "failed"
    OPER_STATE_FAULTY_STATE = "faulty-state"
    OPER_STATE_PENDING_NEXT_BOOT = "pending-next-boot"
    OPER_STATE_PENDING_POWER_CYCLE = "pending-power-cycle"
    OPER_STATE_READY = "ready"
    OPER_STATE_REBOOTING = "rebooting"
    OPER_STATE_REBUILDING = "rebuilding"
    OPER_STATE_SCHEDULED = "scheduled"
    OPER_STATE_SET_STARTUP = "set-startup"
    OPER_STATE_THROTTLED = "throttled"
    OPER_STATE_UPDATING = "updating"
    OPER_STATE_UPGRADING = "upgrading"
    OPER_STATE_QUAL_BOOT_CONF_MISSING = "boot-conf-missing"
    OPER_STATE_QUAL_CHECKSUM_FAILURE = "checksum-failure"
    OPER_STATE_QUAL_CRC_FAILURE = "crc-failure"
    OPER_STATE_QUAL_FILESYSTEM_ERROR = "filesystem-error"
    OPER_STATE_QUAL_MGMT_CONNECT_ERROR = "mgmt-connect-error"
    OPER_STATE_QUAL_NONE = "none"
    OPER_STATE_QUAL_REBUILDING = "rebuilding"
    OPER_STATE_QUAL_UNKNOWN_ERROR = "unknown-error"


class FirmwareUpdatable(ManagedObject):
    """This is FirmwareUpdatable class."""

    consts = FirmwareUpdatableConsts()
    naming_props = set([])

    mo_meta = MoMeta("FirmwareUpdatable", "firmwareUpdatable", "fw-updatable", VersionMeta.Version101e, "InputOutput", 0x7f, [], ["admin"], ['biosUnit', 'equipmentPsu', 'mgmtController', 'storageOnboardDevice'], ['faultInst', 'firmwareInstallable'], ["Get", "Set"])

    prop_meta = {
        "admin_state": MoPropertyMeta("admin_state", "adminState", "string", VersionMeta.Version101e, MoPropertyMeta.READ_WRITE, 0x2, None, None, None, ["force-trigger", "trigger", "triggered"], []),
        "child_action": MoPropertyMeta("child_action", "childAction", "string", VersionMeta.Version101e, MoPropertyMeta.INTERNAL, 0x4, None, None, r"""((deleteAll|ignore|deleteNonPresent),){0,2}(deleteAll|ignore|deleteNonPresent){0,1}""", [], []),
        "deployment": MoPropertyMeta("deployment", "deployment", "string", VersionMeta.Version101e, MoPropertyMeta.READ_ONLY, None, None, None, None, ["backup", "unspecified"], []),
        "dn": MoPropertyMeta("dn", "dn", "string", VersionMeta.Version101e, MoPropertyMeta.READ_ONLY, 0x8, 0, 256, None, [], []),
        "oper_state": MoPropertyMeta("oper_state", "operState", "string", VersionMeta.Version101e, MoPropertyMeta.READ_ONLY, None, None, None, None, ["activating", "auto-activating", "auto-updating", "bad-image", "failed", "faulty-state", "pending-next-boot", "pending-power-cycle", "ready", "rebooting", "rebuilding", "scheduled", "set-startup", "throttled", "updating", "upgrading"], []),
        "oper_state_qual": MoPropertyMeta("oper_state_qual", "operStateQual", "string", VersionMeta.Version101e, MoPropertyMeta.READ_ONLY, None, None, None, None, ["boot-conf-missing", "checksum-failure", "crc-failure", "filesystem-error", "mgmt-connect-error", "none", "rebuilding", "unknown-error"], []),
        "prev_version": MoPropertyMeta("prev_version", "prevVersion", "string", VersionMeta.Version101e, MoPropertyMeta.READ_ONLY, None, 0, 510, None, [], []),
        "rn": MoPropertyMeta("rn", "rn", "string", VersionMeta.Version101e, MoPropertyMeta.READ_ONLY, 0x10, 0, 256, None, [], []),
        "sacl": MoPropertyMeta("sacl", "sacl", "string", VersionMeta.Version302c, MoPropertyMeta.READ_ONLY, None, None, None, r"""((none|del|mod|addchild|cascade),){0,4}(none|del|mod|addchild|cascade){0,1}""", [], []),
        "status": MoPropertyMeta("status", "status", "string", VersionMeta.Version101e, MoPropertyMeta.READ_WRITE, 0x20, None, None, r"""((removed|created|modified|deleted),){0,3}(removed|created|modified|deleted){0,1}""", [], []),
        "version": MoPropertyMeta("version", "version", "string", VersionMeta.Version101e, MoPropertyMeta.READ_WRITE, 0x40, 0, 510, None, [], []),
    }

    prop_map = {
        "adminState": "admin_state", 
        "childAction": "child_action", 
        "deployment": "deployment", 
        "dn": "dn", 
        "operState": "oper_state", 
        "operStateQual": "oper_state_qual", 
        "prevVersion": "prev_version", 
        "rn": "rn", 
        "sacl": "sacl", 
        "status": "status", 
        "version": "version", 
    }

    def __init__(self, parent_mo_or_dn, **kwargs):
        self._dirty_mask = 0
        self.admin_state = None
        self.child_action = None
        self.deployment = None
        self.oper_state = None
        self.oper_state_qual = None
        self.prev_version = None
        self.sacl = None
        self.status = None
        self.version = None

        ManagedObject.__init__(self, "FirmwareUpdatable", parent_mo_or_dn, **kwargs)
